# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import ScenarioTest, record_only


@record_only()
class AzureManagementGroupsScenarioTest(ScenarioTest):
    def test_show_managementgroup(self):
        self.cmd('account management-group create --name testcligetgroup1')
        self.cmd('account management-group create --name testcligetgroup2 --parent /providers/Microsoft.Management/managementGroups/testcligetgroup1')
        managementgroup_get = self.cmd(
            'account management-group show --name testcligetgroup2').get_output_in_json()
        self.cmd('account management-group delete --name testcligetgroup2')
        self.cmd('account management-group delete --name testcligetgroup1')

        self.assertIsNotNone(managementgroup_get)
        self.assertIsNone(managementgroup_get["children"])
        self.assertIsNotNone(managementgroup_get["details"])
        self.assertEqual(
            managementgroup_get["id"],
            "/providers/Microsoft.Management/managementGroups/testcligetgroup2")
        self.assertEqual(managementgroup_get["name"], "testcligetgroup2")
        self.assertEqual(
            managementgroup_get["displayName"],
            "testcligetgroup2")
        self.assertEqual(
            managementgroup_get["details"]["parent"]["displayName"],
            "testcligetgroup1")
        self.assertEqual(
            managementgroup_get["details"]["parent"]["id"],
            "/providers/Microsoft.Management/managementGroups/testcligetgroup1")
        self.assertEqual(
            managementgroup_get["details"]["parent"]["name"],
            "testcligetgroup1")
        self.assertIsNotNone(managementgroup_get["tenantId"])
        self.assertEqual(
            managementgroup_get["type"],
            "/providers/Microsoft.Management/managementGroups")

    def test_show_managementgroup_with_expand(self):
        self.cmd('account management-group create --name testcligetgroup1')
        self.cmd('account management-group create --name testcligetgroup2 --parent testcligetgroup1')
        self.cmd('account management-group create --name testcligetgroup3 --parent /providers/Microsoft.Management/managementGroups/testcligetgroup2')
        managementgroup_get = self.cmd(
            'account management-group show --name testcligetgroup2 --expand').get_output_in_json()
        self.cmd('account management-group delete --name testcligetgroup3')
        self.cmd('account management-group delete --name testcligetgroup2')
        self.cmd('account management-group delete --name testcligetgroup1')

        self.assertIsNotNone(managementgroup_get)
        self.assertIsNotNone(managementgroup_get["children"])
        self.assertIsNotNone(managementgroup_get["details"])
        self.assertEqual(
            managementgroup_get["id"],
            "/providers/Microsoft.Management/managementGroups/testcligetgroup2")
        self.assertEqual(managementgroup_get["name"], "testcligetgroup2")
        self.assertEqual(
            managementgroup_get["displayName"],
            "testcligetgroup2")
        self.assertEqual(
            managementgroup_get["details"]["parent"]["displayName"],
            "testcligetgroup1")
        self.assertEqual(
            managementgroup_get["details"]["parent"]["id"],
            "/providers/Microsoft.Management/managementGroups/testcligetgroup1")
        self.assertEqual(
            managementgroup_get["details"]["parent"]["name"],
            "testcligetgroup1")
        self.assertIsNotNone(managementgroup_get["tenantId"])
        self.assertEqual(
            managementgroup_get["type"],
            "/providers/Microsoft.Management/managementGroups")
        self.assertEqual(
            managementgroup_get["children"][0]["id"],
            "/providers/Microsoft.Management/managementGroups/testcligetgroup3")
        self.assertEqual(
            managementgroup_get["children"][0]["type"],
            "/providers/Microsoft.Management/managementGroups")
        self.assertEqual(
            managementgroup_get["children"][0]["displayName"],
            "testcligetgroup3")
        self.assertEqual(
            managementgroup_get["children"][0]["name"],
            "testcligetgroup3")

    def test_show_managementgroup_with_expand_and_recurse(self):
        self.cmd('account management-group create --name testcligetgroup1')
        self.cmd('account management-group create --name testcligetgroup2 --parent /providers/Microsoft.Management/managementGroups/testcligetgroup1')
        self.cmd('account management-group create --name testcligetgroup3 --parent testcligetgroup2')
        self.cmd('account management-group create --name testcligetgroup4 --parent /providers/Microsoft.Management/managementGroups/testcligetgroup3')
        managementgroup_get = self.cmd(
            'account management-group show --name testcligetgroup2 --expand --recurse').get_output_in_json()
        self.cmd('account management-group delete --name testcligetgroup4')
        self.cmd('account management-group delete --name testcligetgroup3')
        self.cmd('account management-group delete --name testcligetgroup2')
        self.cmd('account management-group delete --name testcligetgroup1')

        self.assertIsNotNone(managementgroup_get)
        self.assertIsNotNone(managementgroup_get["children"])
        self.assertIsNotNone(managementgroup_get["details"])
        self.assertEqual(
            managementgroup_get["id"],
            "/providers/Microsoft.Management/managementGroups/testcligetgroup2")
        self.assertEqual(managementgroup_get["name"], "testcligetgroup2")
        self.assertEqual(
            managementgroup_get["displayName"],
            "testcligetgroup2")
        self.assertEqual(
            managementgroup_get["details"]["parent"]["displayName"],
            "testcligetgroup1")
        self.assertEqual(
            managementgroup_get["details"]["parent"]["id"],
            "/providers/Microsoft.Management/managementGroups/testcligetgroup1")
        self.assertEqual(
            managementgroup_get["details"]["parent"]["name"],
            "testcligetgroup1")
        self.assertIsNotNone(managementgroup_get["tenantId"])
        self.assertEqual(
            managementgroup_get["type"],
            "/providers/Microsoft.Management/managementGroups")
        self.assertEqual(
            managementgroup_get["children"][0]["id"],
            "/providers/Microsoft.Management/managementGroups/testcligetgroup3")
        self.assertEqual(
            managementgroup_get["children"][0]["type"],
            "/providers/Microsoft.Management/managementGroups")
        self.assertEqual(
            managementgroup_get["children"][0]["displayName"],
            "testcligetgroup3")
        self.assertEqual(
            managementgroup_get["children"][0]["name"],
            "testcligetgroup3")
        self.assertEqual(
            managementgroup_get["children"][0]["children"][0]["id"],
            "/providers/Microsoft.Management/managementGroups/testcligetgroup4")
        self.assertEqual(
            managementgroup_get["children"][0]["children"][0]["type"],
            "/providers/Microsoft.Management/managementGroups")
        self.assertEqual(
            managementgroup_get["children"][0]["children"][0]["displayName"],
            "testcligetgroup4")
        self.assertEqual(
            managementgroup_get["children"][0]["children"][0]["name"],
            "testcligetgroup4")

    def test_create_managementgroup(self):
        name = "testcligroup"
        displayName = "testcligroup"
        managementgroup_create = self.cmd(
            'account management-group create --name ' +
            name).get_output_in_json()
        self.cmd('account management-group delete --name ' + name)

        self.assertIsNotNone(managementgroup_create)
        self.assertIsNotNone(managementgroup_create["properties"]["details"])
        self.assertEqual(
            managementgroup_create["id"],
            "/providers/Microsoft.Management/managementGroups/" + name)
        self.assertEqual(managementgroup_create["name"], name)
        self.assertEqual(
            managementgroup_create["properties"]["displayName"],
            displayName)
        self.assertEqual(
            managementgroup_create["properties"]["details"]["parent"]["id"],
            "/providers/Microsoft.Management/managementGroups/" +
            managementgroup_create["properties"]["tenantId"])
        self.assertEqual(
            managementgroup_create["properties"]["details"]["parent"]["name"],
            managementgroup_create["properties"]["tenantId"])
        self.assertIsNotNone(managementgroup_create["properties"]["tenantId"])
        self.assertEqual(
            managementgroup_create["type"],
            "/providers/Microsoft.Management/managementGroups")

    def test_create_managementgroup_with_displayname(self):
        name = "testcligroup"
        displayName = "TestCliDisplayName"
        managementgroup_create = self.cmd(
            'account management-group create --name ' +
            name +
            ' --display-name ' +
            displayName).get_output_in_json()
        self.cmd('account management-group delete --name ' + name)

        self.assertIsNotNone(managementgroup_create)
        self.assertIsNotNone(managementgroup_create["properties"]["details"])
        self.assertEqual(
            managementgroup_create["id"],
            "/providers/Microsoft.Management/managementGroups/" + name)
        self.assertEqual(managementgroup_create["name"], name)
        self.assertEqual(
            managementgroup_create["properties"]["displayName"],
            displayName)
        self.assertEqual(
            managementgroup_create["properties"]["details"]["parent"]["id"],
            "/providers/Microsoft.Management/managementGroups/" +
            managementgroup_create["properties"]["tenantId"])
        self.assertEqual(
            managementgroup_create["properties"]["details"]["parent"]["name"],
            managementgroup_create["properties"]["tenantId"])
        self.assertIsNotNone(managementgroup_create["properties"]["tenantId"])
        self.assertEqual(
            managementgroup_create["type"],
            "/providers/Microsoft.Management/managementGroups")

    def test_create_managementgroup_with_parentid(self):
        name = "testcligroupchild"
        displayName = "testcligroupchild"
        parentId = "/providers/Microsoft.Management/managementGroups/testcligroup"
        parentName = "testcligroup"
        self.cmd('account management-group create --name ' + parentName)
        managementgroup_create = self.cmd(
            'account management-group create --name ' +
            name +
            ' --parent ' +
            parentId).get_output_in_json()
        self.cmd('account management-group delete --name ' + name)
        self.cmd('account management-group delete --name ' + parentName)

        self.assertIsNotNone(managementgroup_create)
        self.assertIsNotNone(managementgroup_create["properties"]["details"])
        self.assertEqual(
            managementgroup_create["id"],
            "/providers/Microsoft.Management/managementGroups/" + name)
        self.assertEqual(managementgroup_create["name"], name)
        self.assertEqual(
            managementgroup_create["properties"]["displayName"],
            displayName)
        self.assertEqual(
            managementgroup_create["properties"]["details"]["parent"]["displayName"],
            parentName)
        self.assertEqual(
            managementgroup_create["properties"]["details"]["parent"]["id"],
            parentId)
        self.assertEqual(
            managementgroup_create["properties"]["details"]["parent"]["name"],
            parentName)
        self.assertIsNotNone(managementgroup_create["properties"]["tenantId"])
        self.assertEqual(
            managementgroup_create["type"],
            "/providers/Microsoft.Management/managementGroups")

    def test_create_managementgroup_with_displayname_and_parentid(self):
        name = "testcligroupchild"
        displayName = "testcligroupchildDisplayName"
        parentId = "/providers/Microsoft.Management/managementGroups/testcligroup"
        parentName = "testcligroup"
        self.cmd('account management-group create --name ' + parentName)
        managementgroup_create = self.cmd(
            'account management-group create --name ' +
            name +
            ' --display-name ' +
            displayName +
            ' --parent ' +
            parentName).get_output_in_json()
        self.cmd('account management-group delete --name ' + name)
        self.cmd('account management-group delete --name ' + parentName)

        self.assertIsNotNone(managementgroup_create)
        self.assertIsNotNone(managementgroup_create["properties"]["details"])
        self.assertEqual(
            managementgroup_create["id"],
            "/providers/Microsoft.Management/managementGroups/" + name)
        self.assertEqual(managementgroup_create["name"], name)
        self.assertEqual(
            managementgroup_create["properties"]["displayName"],
            displayName)
        self.assertEqual(
            managementgroup_create["properties"]["details"]["parent"]["displayName"],
            parentName)
        self.assertEqual(
            managementgroup_create["properties"]["details"]["parent"]["id"],
            parentId)
        self.assertEqual(
            managementgroup_create["properties"]["details"]["parent"]["name"],
            parentName)
        self.assertIsNotNone(managementgroup_create["properties"]["tenantId"])
        self.assertEqual(
            managementgroup_create["type"],
            "/providers/Microsoft.Management/managementGroups")

    def test_update_managementgroup_with_displayname(self):
        name = "testcligroup"
        displayName = "testcligroupDisplayName"
        self.cmd('account management-group create --name ' + name)
        managementgroup_update = self.cmd(
            'account management-group update --name ' +
            name +
            ' --display-name ' +
            displayName).get_output_in_json()
        self.cmd('account management-group delete --name ' + name)

        self.assertIsNotNone(managementgroup_update)
        self.assertIsNotNone(managementgroup_update["details"])
        self.assertEqual(
            managementgroup_update["id"],
            "/providers/Microsoft.Management/managementGroups/" + name)
        self.assertEqual(managementgroup_update["name"], name)
        self.assertEqual(managementgroup_update["displayName"], displayName)
        self.assertEqual(
            managementgroup_update["details"]["parent"]["id"],
            "/providers/Microsoft.Management/managementGroups/" +
            managementgroup_update["tenantId"])
        self.assertEqual(
            managementgroup_update["details"]["parent"]["name"],
            managementgroup_update["tenantId"])
        self.assertIsNotNone(managementgroup_update["tenantId"])
        self.assertEqual(
            managementgroup_update["type"],
            "/providers/Microsoft.Management/managementGroups")

    def test_update_managementgroup_with_parentid(self):
        name = "testcligroupchild"
        displayName = "testcligroupchild"
        parentId = "/providers/Microsoft.Management/managementGroups/testcligroup"
        parentName = "testcligroup"
        self.cmd('account management-group create --name ' + parentName)
        self.cmd('account management-group create --name ' + name)
        managementgroup_update = self.cmd(
            'account management-group update --name ' +
            name +
            ' --parent ' +
            parentId).get_output_in_json()
        self.cmd('account management-group delete --name ' + name)
        self.cmd('account management-group delete --name ' + parentName)

        self.assertIsNotNone(managementgroup_update)
        self.assertIsNotNone(managementgroup_update["details"])
        self.assertEqual(
            managementgroup_update["id"],
            "/providers/Microsoft.Management/managementGroups/" + name)
        self.assertEqual(managementgroup_update["name"], name)
        self.assertEqual(managementgroup_update["displayName"], displayName)
        self.assertEqual(
            managementgroup_update["details"]["parent"]["displayName"],
            parentName)
        self.assertEqual(
            managementgroup_update["details"]["parent"]["id"],
            parentId)
        self.assertEqual(
            managementgroup_update["details"]["parent"]["name"],
            parentName)
        self.assertIsNotNone(managementgroup_update["tenantId"])
        self.assertEqual(
            managementgroup_update["type"],
            "/providers/Microsoft.Management/managementGroups")

    def test_update_managementgroup_with_displayname_and_parentid(self):
        name = "testcligroupchild"
        displayName = "testcligroupchild"
        parentId = "/providers/Microsoft.Management/managementGroups/testcligroup"
        parentName = "testcligroup"
        self.cmd('account management-group create --name ' + parentName)
        self.cmd('account management-group create --name ' + name)
        managementgroup_update = self.cmd(
            'account management-group update --name ' +
            name +
            ' --display-name ' +
            displayName +
            ' --parent ' +
            parentName).get_output_in_json()
        self.cmd('account management-group delete --name ' + name)
        self.cmd('account management-group delete --name ' + parentName)

        self.assertIsNotNone(managementgroup_update)
        self.assertIsNotNone(managementgroup_update["details"])
        self.assertEqual(
            managementgroup_update["id"],
            "/providers/Microsoft.Management/managementGroups/" + name)
        self.assertEqual(managementgroup_update["name"], name)
        self.assertEqual(managementgroup_update["displayName"], displayName)
        self.assertEqual(
            managementgroup_update["details"]["parent"]["displayName"],
            parentName)
        self.assertEqual(
            managementgroup_update["details"]["parent"]["id"],
            parentId)
        self.assertEqual(
            managementgroup_update["details"]["parent"]["name"],
            parentName)
        self.assertIsNotNone(managementgroup_update["tenantId"])
        self.assertEqual(
            managementgroup_update["type"],
            "/providers/Microsoft.Management/managementGroups")

    def test_create_delete_group_managementgroup(self):
        self.cmd('account management-group create --name testcligroup')
        self.cmd('account management-group delete --name testcligroup')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest
from azure.cli.command_modules.servicefabric.tests.latest.test_util import (
    _create_keyvault,
    _add_selfsigned_cert_to_keyvault
)
from azure.cli.core.util import CLIError
from azure.cli.testsdk import ScenarioTest, LiveScenarioTest, ResourceGroupPreparer
from azure.mgmt.servicefabric.models import ErrorModelException


class ServiceFabricManagedClustersTests(ScenarioTest):
    @ResourceGroupPreparer()
    def test_basic_cluster(self):
        self.kwargs.update({
            'cert_tp': '123BDACDCDFB2C7B250192C6078E47D1E1DB119B',
            'loc': 'eastasia',
            'cluster_name': self.create_random_name('sfrp-cli-', 24),
            'vm_password': self.create_random_name('Pass@', 9)
        })

        self.cmd('az sf managed-cluster create -g {rg} -c {cluster_name} -l {loc} --cert-thumbprint {cert_tp} --cert-is-admin --admin-password {vm_password}',
                 checks=[self.check('provisioningState', 'Succeeded'),
                         self.check('clusterState', 'WaitingForNodes')])

        self.cmd('az sf managed-node-type create -g {rg} -c {cluster_name} -n pnt --instance-count 5 --primary',
                 checks=[self.check('provisioningState', 'Succeeded')])

        # 'InvalidParameter - Cluster must have at least one active primary node type'
        with self.assertRaisesRegexp(ErrorModelException, 'Cluster must have at least one active primary node type'):
            self.cmd('az sf managed-node-type delete -g {rg} -c {cluster_name} -n pnt')

        self.cmd('az sf managed-cluster show -g {rg} -c {cluster_name}',
                 checks=[self.check('clusterState', 'Deploying')])

        self.cmd('az sf managed-cluster delete -g {rg} -c {cluster_name}')

        # SystemExit 3 'not found'
        with self.assertRaisesRegexp(SystemExit, '3'):
            self.cmd('az sf managed-cluster show -g {rg} -c {cluster_name}')

    @ResourceGroupPreparer()
    def test_node_type_operation(self):
        self.kwargs.update({
            'cert_tp': '123BDACDCDFB2C7B250192C6078E47D1E1DB119B',
            'loc': 'eastasia',
            'cluster_name': self.create_random_name('sfrp-cli-', 24),
            'vm_password': self.create_random_name('Pass@', 9)
        })

        self.cmd('az sf managed-cluster create -g {rg} -c {cluster_name} -l {loc} --cert-thumbprint {cert_tp} --cert-is-admin --admin-password {vm_password} --sku Standard',
                 checks=[self.check('provisioningState', 'Succeeded'),
                         self.check('clusterState', 'WaitingForNodes')])

        self.cmd('az sf managed-node-type create -g {rg} -c {cluster_name} -n pnt --instance-count 5 --primary',
                 checks=[self.check('provisioningState', 'Succeeded')])

        self.cmd('az sf managed-node-type list -g {rg} -c {cluster_name}',
                 checks=[self.check('length(@)', 1)])

        self.cmd('az sf managed-node-type create -g {rg} -c {cluster_name} -n snt --instance-count 6',
                 checks=[self.check('provisioningState', 'Succeeded')])

        self.cmd('az sf managed-node-type list -g {rg} -c {cluster_name}',
                 checks=[self.check('length(@)', 2)])

        self.cmd('az sf managed-node-type node restart -g {rg} -c {cluster_name} -n snt --node-name snt_0 snt_1')

        self.cmd('az sf managed-node-type node delete -g {rg} -c {cluster_name} -n snt --node-name snt_1')

        self.cmd('az sf managed-node-type node reimage -g {rg} -c {cluster_name} -n snt --node-name snt_3')

        self.cmd('az sf managed-node-type delete -g {rg} -c {cluster_name} -n snt')

        # SystemExit 3 'not found'
        with self.assertRaisesRegexp(SystemExit, '3'):
            self.cmd('az sf managed-node-type show -g {rg} -c {cluster_name} -n snt')

        self.cmd('az sf managed-node-type list -g {rg} -c {cluster_name}',
                 checks=[self.check('length(@)', 1)])

        self.cmd('az sf managed-cluster delete -g {rg} -c {cluster_name}')

        # SystemExit 3 'not found'
        with self.assertRaisesRegexp(SystemExit, '3'):
            self.cmd('az sf managed-cluster show -g {rg} -c {cluster_name}')

    @ResourceGroupPreparer()
    def test_cert_and_ext(self):
        self.kwargs.update({
            'cert_tp': '123BDACDCDFB2C7B250192C6078E47D1E1DB119B',
            'cert_tp2': '123BDACDCDFB2C7B250192C6078E47D1E1DB7777',
            'loc': 'eastasia',
            'cluster_name': self.create_random_name('sfrp-cli-', 24),
            'vm_password': self.create_random_name('Pass@', 9),
            'extName': 'csetest',
            'publisher': 'Microsoft.Compute',
            'extType': 'BGInfo',
            'extVer': '2.1',
            'kv_name': self.create_random_name('sfrp-cli-kv-', 24),
            'cert_name': self.create_random_name('sfrp-cli-', 24)
        })

        self.cmd('az sf managed-cluster create -g {rg} -c {cluster_name} -l {loc} --cert-thumbprint {cert_tp} --cert-is-admin --admin-password {vm_password}',
                 checks=[self.check('provisioningState', 'Succeeded'),
                         self.check('clusterState', 'WaitingForNodes')])

        self.cmd('az sf managed-node-type create -g {rg} -c {cluster_name} -n pnt --instance-count 5 --primary',
                 checks=[self.check('provisioningState', 'Succeeded')])

        # add extension
        self.cmd('az sf managed-node-type vm-extension add -g {rg} -c {cluster_name} -n pnt '
                 ' --extension-name {extName} --publisher {publisher} --extension-type {extType} --type-handler-version {extVer} --auto-upgrade-minor-version',
                 checks=[self.check('provisioningState', 'Succeeded')])

        self.cmd('az sf managed-node-type show -g {rg} -c {cluster_name} -n pnt',
                 checks=[self.check('length(vmExtensions)', 1)])

        # add secret
        kv = _create_keyvault(self, self.kwargs)
        self.kwargs.update({'kv_id': kv['id']})
        cert = _add_selfsigned_cert_to_keyvault(self, self.kwargs)
        cert_secret_id = cert['sid']
        self.kwargs.update({'cert_secret_id': cert_secret_id})

        self.cmd('az sf managed-node-type vm-secret add -g {rg} -c {cluster_name} -n pnt '
                 ' --source-vault-id {kv_id} --certificate-url {cert_secret_id} --certificate-store my',
                 checks=[self.check('provisioningState', 'Succeeded')])

        self.cmd('az sf managed-node-type show -g {rg} -c {cluster_name} -n pnt',
                 checks=[self.check('length(vmSecrets)', 1)])

        # add client cert
        self.cmd('az sf managed-cluster client-certificate add -g {rg} -c {cluster_name} --thumbprint {cert_tp2}',
                 checks=[self.check('provisioningState', 'Succeeded')])

        self.cmd('az sf managed-cluster show -g {rg} -c {cluster_name}',
                 checks=[self.check('length(clients)', 2)])

        # delete client cert
        self.cmd('az sf managed-cluster client-certificate delete -g {rg} -c {cluster_name} --thumbprint {cert_tp2}',
                 checks=[self.check('provisioningState', 'Succeeded')])

        self.cmd('az sf managed-cluster show -g {rg} -c {cluster_name}',
                 checks=[self.check('length(clients)', 1)])

        self.cmd('az sf managed-cluster delete -g {rg} -c {cluster_name}')

        # SystemExit 3 'not found'
        with self.assertRaisesRegexp(SystemExit, '3'):
            self.cmd('az sf managed-cluster show -g {rg} -c {cluster_name}')


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest
import mock

from azure.cli.command_modules.vm._template_builder import build_load_balancer_resource


class TestTemplateBuilder(unittest.TestCase):

    @mock.patch('azure.cli.command_modules.vm._template_builder.get_target_network_api', autospec=True)
    def test_build_load_balancer_resource(self, mock_get_api):
        mock_get_api.returtn_value = '1970-01-01'
        cmd_mock = mock.MagicMock()
        cmd_mock.supported_api_version.return_value = False

        result = build_load_balancer_resource(cmd_mock, 'lb1', 'westus', None, 'bepool1', 'natpool1', 'be_port',
                                              'frontip', 'pip_id1', 'subnet_id1', 'private_ip_address', 'dynamic',
                                              'basic', instance_count=1, disable_overprovision=False)
        self.assertEqual(result['properties']['inboundNatPools'][0]['properties']['frontendPortRangeEnd'], '50119')

        result = build_load_balancer_resource(cmd_mock, 'lb1', 'westus', None, 'bepool1', 'natpool1', 'be_port',
                                              'frontip', 'pip_id1', 'subnet_id1', 'private_ip_address', 'dynamic',
                                              'basic', instance_count=80, disable_overprovision=False)
        self.assertEqual(result['properties']['inboundNatPools'][0]['properties']['frontendPortRangeEnd'], '50159')

        result = build_load_balancer_resource(cmd_mock, 'lb1', 'westus', None, 'bepool1', 'natpool1', 'be_port',
                                              'frontip', 'pip_id1', 'subnet_id1', 'private_ip_address', 'dynamic',
                                              'basic', instance_count=80, disable_overprovision=True)
        self.assertEqual(result['properties']['inboundNatPools'][0]['properties']['frontendPortRangeEnd'], '50119')

        result = build_load_balancer_resource(cmd_mock, 'lb1', 'westus', None, 'bepool1', 'natpool1', 'be_port',
                                              'frontip', 'pip_id1', 'subnet_id1', 'private_ip_address', 'dynamic',
                                              'basic', instance_count=140, disable_overprovision=True)
        self.assertEqual(result['properties']['inboundNatPools'][0]['properties']['frontendPortRangeEnd'], '50139')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import logging
import unittest
import shlex
import sys
from msrest.serialization import Model
from azure.cli.core import AzCommandsLoader

from azure.cli.core.mock import DummyCli

from knack.util import CLIError


class ListTestObject(object):

    def __init__(self, val):
        self.list_value = list(val)


class DictTestObject(object):

    def __init__(self, val):
        self.dict_value = dict(val)


class ObjectTestObject(object):

    def __init__(self, str_val, int_val, bool_val):
        self.my_string = str(str_val)
        self.my_int = int(int_val)
        self.my_bool = bool(bool_val)
        self.additional_properties = None


class ComplexTestObject(Model):

    def __init__(self):
        self.my_prop = 'my_value'
        self.my_list = [
            'myValA',
            ['myValB', 'myValC'],
            {'myKey': 'valueA'},
            ObjectTestObject('myString', 0, True)
        ]
        self.my_dict = {}
        self.my_list_of_camel_dicts = [
            {'myKey': 'value_1'},
            {'myKey': 'value_2'}
        ]
        self.my_list_of_snake_dicts = [
            {'my_key': 'value1'},
            {'my_key': 'value2'}
        ]
        self.my_list_of_objects = [
            ObjectTestObject('myKeyA', 25, True),
            ObjectTestObject('1.2.3.4', 100, False),
        ]
        self.empty_prop = None
        self.empty_list = []
        self.empty_dict_of_dicts = {'dict': {'dict2': None}}
        self.empty_dict = {'dict3': None}

        self.additional_properties = {'additional1': 'addition value #1', 'additionalList': []}
        self.my_test_object = ObjectTestObject('myKeyAA', 1, False)


def _prepare_test_loader():

    my_obj = ComplexTestObject()

    class GenericUpdateTestCommandsLoader(AzCommandsLoader):

        def load_command_table(self, args):
            super(GenericUpdateTestCommandsLoader, self).load_command_table(args)

            from azure.cli.core.commands import CliCommandType

            def my_get():
                return my_obj

            def my_set(**kwargs):  # pylint:disable=unused-argument
                return my_obj

            test_module = 'azure.cli.core.tests.test_generic_update'
            test_type = CliCommandType(operations_tmpl='{}#{{}}'.format(test_module))
            try:
                setattr(sys.modules[test_module], my_get.__name__, my_get)
                setattr(sys.modules[test_module], my_set.__name__, my_set)
            except KeyError:
                import importlib
                loaded_module = importlib.import_module(test_module)
                setattr(loaded_module, my_get.__name__, my_get)
                setattr(loaded_module, my_set.__name__, my_set)
            with self.command_group('', test_type) as g:
                g.generic_update_command('genupdate', getter_name='my_get', setter_name='my_set')

            return self.command_table
    return my_obj, GenericUpdateTestCommandsLoader


class GenericUpdateTest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        logging.getLogger().setLevel(logging.ERROR)

    def test_generic_update_scenario(self):  # pylint: disable=too-many-statements

        my_obj, loader_cls = _prepare_test_loader()
        cli = DummyCli(commands_loader_cls=loader_cls)

        # Test simplest ways of setting properties
        cli.invoke('genupdate --set myProp=newValue'.split())
        self.assertEqual(my_obj.my_prop, 'newValue', 'set simple property')

        cli.invoke('genupdate --set myProp=val3'.split())
        self.assertEqual(my_obj.my_prop, 'val3', 'set simple property again')

        cli.invoke('genupdate --set myProp="foo=bar"'.split())
        self.assertEqual(my_obj.my_prop, 'foo=bar', 'use equal in value')

        cli.invoke('genupdate --set myList[0]=newValA'.split())
        self.assertEqual(my_obj.my_list[0], 'newValA', 'set simple list element')

        cli.invoke('genupdate --set myList[-4]=newValB'.split())
        self.assertEqual(my_obj.my_list[0], 'newValB', 'set simple list element')

        cli.invoke('genupdate --set myDict.myCamelKey=success'.split())
        self.assertEqual(my_obj.my_dict['myCamelKey'], 'success',
                         'set simple dict element with camel case key')

        cli.invoke('genupdate --set myDict.my_snake_key=success'.split())
        self.assertEqual(my_obj.my_dict['my_snake_key'], 'success',
                         'set simple dict element with snake case key')

        cli.invoke('genupdate --set additional1=v1'.split())
        self.assertEqual(my_obj.additional_properties['additional1'], 'v1')

        cli.invoke('genupdate --set my_test_object.additional1=v1'.split())  # for unknown properties, we also set as additional_proeprties
        self.assertEqual(my_obj.my_test_object.additional_properties['additional1'], 'v1')

        # Test the different ways of indexing into a list of objects or dictionaries by filter
        cli.invoke('genupdate --set myListOfCamelDicts[myKey=value_2].myKey="foo=bar"'.split())
        self.assertEqual(my_obj.my_list_of_camel_dicts[1]['myKey'],
                         'foo=bar',
                         'index into list of dictionaries by camel-case key and set value with =')

        cli.invoke('genupdate --set myListOfCamelDicts[myKey="foo=bar"].myKey=new_value'.split())
        self.assertEqual(my_obj.my_list_of_camel_dicts[1]['myKey'],
                         'new_value',
                         'index into list of dictionaries by camel-case key')

        cli.invoke('genupdate --set myListOfSnakeDicts[my_key=value2].my_key=new_value'.split())
        self.assertEqual(my_obj.my_list_of_snake_dicts[1]['my_key'],
                         'new_value',
                         'index into list of dictionaries by snake-case key')

        cli.invoke('genupdate --set myListOfObjects[myString=1.2.3.4].myString=new_value'.split())
        self.assertEqual(my_obj.my_list_of_objects[1].my_string,
                         'new_value',
                         'index into list of objects by key')

        # Test setting on elements nested within lists
        cli.invoke('genupdate --set myList[1][1]=newValue'.split())
        self.assertEqual(my_obj.my_list[1][1], 'newValue', 'set nested list element')

        cli.invoke('genupdate --set myList[2].myKey=newValue'.split())
        self.assertEqual(my_obj.my_list[2]['myKey'], 'newValue', 'set nested dict element')

        cli.invoke('genupdate --set myList[3].myInt=50'.split())
        self.assertEqual(my_obj.my_list[3].my_int, 50, 'set nested object element')

        # Test overwriting and removing values
        cli.invoke('genupdate --set myProp={} myProp.foo=bar'.split())
        self.assertEqual(my_obj.my_prop['foo'], 'bar', 'replace scalar with dict')

        cli.invoke(
            'genupdate --set myProp=[] --add myProp key1=value1 --set myProp[0].key2=value2'.split())
        self.assertEqual(my_obj.my_prop[0]['key1'], 'value1',
                         'replace scalar with new list and add a dict entry')
        self.assertEqual(my_obj.my_prop[0]['key2'], 'value2',
                         'add a second value to the new dict entry')

        cli.invoke('genupdate --remove myProp --add myProp str1 str2 --remove myProp 0'.split())
        self.assertEqual(len(my_obj.my_prop), 1, 'nullify property, add two and remove one')
        self.assertEqual(my_obj.my_prop[0], 'str2', 'nullify property, add two and remove one')

        cli.invoke('genupdate --add additionalList listMember1'.split())
        self.assertEqual(my_obj.additional_properties['additionalList'][0], 'listMember1',
                         'add a value to an array inside additional_properties')
        # Test various --add to lists
        cli.invoke('genupdate --set myList=[]'.split())
        cli.invoke(shlex.split('genupdate --add myList value1'))
        self.assertEqual(my_obj.my_list[0], 'value1', 'add a value to an array')

        cli.invoke('genupdate --set myList=[]'.split())
        cli.invoke(shlex.split(
            'genupdate --add myList key1=value1 key2=value2 foo "string in quotes" [] {} foo=bar'))
        self.assertEqual(my_obj.my_list[0]['key1'], 'value1', 'add a value to a dictionary')
        self.assertEqual(my_obj.my_list[0]['key2'], 'value2',
                         'add a second value to the dictionary')
        self.assertEqual(my_obj.my_list[1], 'foo', 'add scalar value to list')
        self.assertEqual(my_obj.my_list[2], 'string in quotes',
                         'add scalar value with quotes to list')
        self.assertEqual(my_obj.my_list[3], [], 'add list to a list')
        self.assertEqual(my_obj.my_list[4], {}, 'add dict to a list')
        self.assertEqual(my_obj.my_list[-1]['foo'], 'bar',
                         'add second dict and verify when dict is at the end')

        # Test --remove
        self.assertEqual(len(my_obj.my_list), 6, 'pre-verify length of list')
        cli.invoke('genupdate --remove myList -2'.split())
        self.assertEqual(len(my_obj.my_list), 5, 'verify one item removed')
        self.assertEqual(my_obj.my_list[4]['foo'], 'bar', 'verify correct item removed')
        self.assertEqual('key1' in my_obj.my_list[0], True, 'verify dict item exists')

        cli.invoke('genupdate --remove myList[0].key1'.split())
        self.assertEqual('key1' not in my_obj.my_list[0], True, 'verify dict entry can be removed')

        cli.invoke('genupdate --remove myList'.split())
        self.assertEqual(my_obj.my_list, [])

    def test_generic_update_errors(self):  # pylint: disable=no-self-use

        my_obj, loader_cls = _prepare_test_loader()
        cli = DummyCli(commands_loader_cls=loader_cls)

        def _execute_with_error(command, error, message):
            try:
                cli.invocation = cli.invocation_cls(cli_ctx=cli, parser_cls=cli.parser_cls,
                                                    commands_loader_cls=cli.commands_loader_cls, help_cls=cli.help_cls)
                cli.invocation.execute(command.split())
            except CLIError as ex:
                if error not in str(ex):
                    raise AssertionError('{}\nExpected: {}\nActual: {}'.format(message, error, ex))
                return
            except Exception as ex:
                raise ex
            raise AssertionError("exception not raised for '{0}'".format(message))

        missing_remove_message = "Couldn't find 'doesntExist' in ''. Available options: ['additional1', 'additionalList', 'emptyDict', 'emptyDictOfDicts', 'emptyList', 'emptyProp', 'myDict', 'myList', 'myListOfCamelDicts', 'myListOfObjects', 'myListOfSnakeDicts', 'myProp', 'myTestObject']"
        _execute_with_error('genupdate --remove doesntExist',
                            missing_remove_message,
                            'remove non-existent property by name')
        _execute_with_error('genupdate --remove doesntExist 2',
                            missing_remove_message,
                            'remove non-existent property by index')

        remove_prop_message = "Couldn't find 'doesntExist' in 'myList.doesntExist'. Available options: index into the collection 'myList.doesntExist' with [<index>] or [<key=value>]"
        _execute_with_error('genupdate --remove myList.doesntExist.missing 2',
                            remove_prop_message,
                            'remove non-existent sub-property by index')

        _execute_with_error('genupdate --remove myList 20',
                            "index 20 doesn't exist on myList",
                            'remove out-of-range index')

        set_on_list_message = "Couldn't find 'doesnt_exist' in 'myList'. Available options: index into the collection 'myList' with [<index>] or [<key=value>]"
        _execute_with_error('genupdate --set myList.doesnt_exist=foo',
                            set_on_list_message,
                            'set shouldn\'t work on a list')
        _execute_with_error('genupdate --set myList.doesnt_exist.doesnt_exist2=foo',
                            set_on_list_message,
                            'set shouldn\'t work on a list')

        _execute_with_error('genupdate --set myList[5].doesnt_exist=foo',
                            "index 5 doesn't exist on myList",
                            'index out of range in path')

        _execute_with_error('genupdate --remove myList[0]',
                            'invalid syntax: --remove property.list <indexToRemove> OR --remove propertyToRemove',
                            'remove requires index to be space-separated')

        cli.invoke("genupdate --set myDict={'foo':'bar'}".split())
        _execute_with_error('genupdate --set myDict.foo.doo=boo',
                            "Couldn't find 'doo' in 'myDict.foo'. 'myDict.foo' does not support further indexing.",
                            'Cannot dot index from a scalar value')

        _execute_with_error('genupdate --set myDict.foo[0]=boo',
                            "Couldn't find '[0]' in 'myDict'. 'myDict' does not support further indexing.",
                            'Cannot list index from a scalar value')

        _execute_with_error('genupdate --add myDict la=da',
                            "invalid syntax: --add property.listProperty <key=value, string or JSON string>",
                            'Add only works with lists')

        # add an entry which makes 'myKey' no longer unique
        cli.invoke('genupdate --add myListOfCamelDicts myKey=value_2'.split())
        _execute_with_error(
            'genupdate --set myListOfCamelDicts[myKey=value_2].myKey=foo',
            "non-unique key 'myKey' found multiple matches on myListOfCamelDicts. "
            "Key must be unique.",
            'indexing by key must be unique')

        _execute_with_error(
            'genupdate --set myListOfCamelDicts[myKey=foo].myKey=foo',
            "item with value 'foo' doesn\'t exist for key 'myKey' on myListOfCamelDicts",
            'no match found when indexing by key and value')

    def test_generic_update_empty_nodes(self):

        my_obj, loader_cls = _prepare_test_loader()
        cli = DummyCli(commands_loader_cls=loader_cls)

        # add to prop
        cli.invoke('genupdate --add emptyProp a=b'.split())
        self.assertEqual(my_obj.empty_prop[0]['a'], 'b', 'verify object added to null list')
        self.assertEqual(len(my_obj.empty_prop[0]), 1, 'verify only one object added to null list')

        # add to list
        cli.invoke('genupdate --add emptyList c=d'.split())
        self.assertEqual(my_obj.empty_list[0]['c'], 'd', 'verify object added to empty list')
        self.assertEqual(len(my_obj.empty_list), 1, 'verify only one object added to empty list')

        # set dict2
        cli.invoke('genupdate --set emptyDictOfDicts.dict.dict2.e=f'.split())
        self.assertEqual(my_obj.empty_dict_of_dicts['dict']['dict2']['e'], 'f', 'verify object added to null dict')
        self.assertEqual(len(my_obj.empty_dict_of_dicts['dict']['dict2']), 1,
                         'verify only one object added to null dict')

        # set dict3
        cli.invoke('genupdate --set emptyDict.dict3.g=h'.split())
        self.assertEqual(my_obj.empty_dict['dict3']['g'], 'h', 'verify object added to empty dict')
        self.assertEqual(len(my_obj.empty_dict['dict3']), 1, 'verify only one object added to empty dict')


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import json
from datetime import datetime, timedelta
import unittest

from azure.cli.testsdk import ScenarioTest, JMESPathCheckExists, ResourceGroupPreparer, \
    StorageAccountPreparer, record_only
from azure.mgmt.recoveryservicesbackup.models import StorageType
from azure_devtools.scenario_tests import AllowLargeResponse

from .preparers import VaultPreparer, VMPreparer, ItemPreparer, PolicyPreparer, RPPreparer


def _get_vm_version(vm_type):
    if vm_type == 'Microsoft.Compute/virtualMachines':
        return 'Compute'
    elif vm_type == 'Microsoft.ClassicCompute/virtualMachines':
        return 'Classic'


class BackupTests(ScenarioTest, unittest.TestCase):
    @ResourceGroupPreparer(location="southeastasia")
    @VaultPreparer()
    @VMPreparer()
    @StorageAccountPreparer(location="southeastasia")
    def test_backup_scenario(self, resource_group, vault_name, vm_name, storage_account):

        self.kwargs.update({
            'vault': vault_name,
            'vm': vm_name
        })

        # Enable Protection
        self.cmd('backup protection enable-for-vm -g {rg} -v {vault} --vm {vm} -p DefaultPolicy').get_output_in_json()

        # Get Container
        self.kwargs['container'] = self.cmd('backup container show -n {vm} -v {vault} -g {rg} --backup-management-type AzureIaasVM --query properties.friendlyName').get_output_in_json()

        # Get Item
        self.kwargs['item'] = self.cmd('backup item list -g {rg} -v {vault} -c {container} --backup-management-type AzureIaasVM --workload-type VM --query [0].properties.friendlyName').get_output_in_json()

        # Trigger Backup
        self.kwargs['retain_date'] = (datetime.utcnow() + timedelta(days=30)).strftime('%d-%m-%Y')
        self.kwargs['job'] = self.cmd('backup protection backup-now -g {rg} -v {vault} -c {container} -i {item} --backup-management-type AzureIaasVM --workload-type VM --retain-until {retain_date} --query name').get_output_in_json()
        self.cmd('backup job wait -g {rg} -v {vault} -n {job}')

        # Get Recovery Point
        self.kwargs['recovery_point'] = self.cmd('backup recoverypoint list -g {rg} -v {vault} -c {container} -i {item} --backup-management-type AzureIaasVM --workload-type VM --query [0].name').get_output_in_json()

        # Trigger Restore
        self.kwargs['job'] = self.cmd('backup restore restore-disks -g {rg} -v {vault} -c {container} -i {item} -r {recovery_point} --storage-account {sa} --query name --restore-to-staging-storage-account').get_output_in_json()
        self.cmd('backup job wait -g {rg} -v {vault} -n {job}')

        # Disable protection with retain data
        self.cmd('backup protection disable -g {rg} -v {vault} -c {container} -i {item} --backup-management-type AzureIaasVM --workload-type VM --yes')

        # Resume protection
        self.cmd('backup protection resume -g {rg} -v {vault} -c {container} -i {item} --policy-name DefaultPolicy --backup-management-type AzureIaasVM', checks=[
            self.check("properties.entityFriendlyName", '{item}'),
            self.check("properties.operation", "ConfigureBackup"),
            self.check("properties.status", "Completed"),
            self.check("resourceGroup", '{rg}')
        ])

        # Disable Protection with delete data
        self.cmd('backup protection disable -g {rg} -v {vault} -c {container} -i {item} --backup-management-type AzureIaasVM --workload-type VM --delete-backup-data true --yes')

    @record_only()
    @ResourceGroupPreparer()
    @VaultPreparer(parameter_name='vault1')
    @VaultPreparer(parameter_name='vault2')
    def test_backup_vault(self, resource_group, resource_group_location, vault1, vault2):

        self.kwargs.update({
            'loc': resource_group_location,
            'vault1': vault1,
            'vault2': vault2
        })

        self.kwargs['vault3'] = self.create_random_name('clitest-vault', 50)
        self.cmd('backup vault create -n {vault3} -g {rg} -l {loc}', checks=[
            self.check('name', '{vault3}'),
            self.check('resourceGroup', '{rg}'),
            self.check('location', '{loc}'),
            self.check('properties.provisioningState', 'Succeeded')
        ])

        self.kwargs['vault4'] = self.create_random_name('clitest-vault', 50)
        self.cmd('backup vault create -n {vault4} -g {rg} -l {loc}', checks=[
            self.check('name', '{vault4}'),
            self.check('resourceGroup', '{rg}'),
            self.check('location', '{loc}'),
            self.check('properties.provisioningState', 'Succeeded')
        ])

        number_of_test_vaults = 4

        self.cmd('backup vault list', checks=[
            self.check("length([?resourceGroup == '{rg}'])", number_of_test_vaults),
            self.check("length([?name == '{vault1}'])", 1),
            self.check("length([?name == '{vault2}'])", 1),
            self.check("length([?name == '{vault3}'])", 1),
            self.check("length([?name == '{vault4}'])", 1)
        ])

        self.cmd('backup vault list -g {rg}', checks=[
            self.check("length(@)", number_of_test_vaults),
            self.check("length([?name == '{vault1}'])", 1),
            self.check("length([?name == '{vault2}'])", 1),
            self.check("length([?name == '{vault3}'])", 1),
            self.check("length([?name == '{vault4}'])", 1)
        ])

        storage_model_types = [e.value for e in StorageType]
        vault_properties = self.cmd('backup vault backup-properties show -n {vault1} -g {rg} --query [0]', checks=[
            JMESPathCheckExists("contains({}, properties.storageModelType)".format(storage_model_types)),
            self.check('properties.storageTypeState', 'Unlocked'),
            self.check('resourceGroup', '{rg}')
        ]).get_output_in_json()

        if vault_properties['properties']['storageModelType'] == StorageType.geo_redundant.value:
            new_storage_model = StorageType.locally_redundant.value
        else:
            new_storage_model = StorageType.geo_redundant.value

        self.kwargs['model'] = new_storage_model
        self.cmd('backup vault backup-properties set -n {vault1} -g {rg} --backup-storage-redundancy {model}')

        self.cmd('backup vault backup-properties show -n {vault1} -g {rg} --query [0]', checks=[
            self.check('properties.storageModelType', new_storage_model)
        ])

        self.cmd('backup vault delete -n {vault4} -g {rg} -y')

        self.cmd('backup vault list', checks=[
            self.check("length([?resourceGroup == '{rg}'])", number_of_test_vaults - 1),
            self.check("length([?name == '{vault1}'])", 1),
            self.check("length([?name == '{vault2}'])", 1),
            self.check("length([?name == '{vault3}'])", 1)
        ])

    @ResourceGroupPreparer(location="southeastasia")
    @VaultPreparer()
    @VMPreparer(parameter_name='vm1')
    @VMPreparer(parameter_name='vm2')
    @ItemPreparer(vm_parameter_name='vm1')
    @ItemPreparer(vm_parameter_name='vm2')
    def test_backup_container(self, resource_group, vault_name, vm1, vm2):

        self.kwargs.update({
            'vault': vault_name,
            'vm1': vm1,
            'vm2': vm2
        })

        container_json = self.cmd('backup container show --backup-management-type AzureIaasVM -n {vm1} -v {vault} -g {rg}', checks=[
            self.check('properties.friendlyName', '{vm1}'),
            self.check('properties.healthStatus', 'Healthy'),
            self.check('properties.registrationStatus', 'Registered'),
            self.check('properties.resourceGroup', '{rg}'),
            self.check('resourceGroup', '{rg}')
        ]).get_output_in_json()

        vm1_json = self.cmd('vm show -n {vm1} -g {rg}').get_output_in_json()

        self.assertIn(vault_name.lower(), container_json['id'].lower())
        self.assertIn(vm1.lower(), container_json['name'].lower())
        self.assertIn(vm1.lower(), container_json['properties']['virtualMachineId'].lower())
        self.assertEqual(container_json['properties']['virtualMachineVersion'], _get_vm_version(vm1_json['type']))

        self.cmd('backup container list --backup-management-type AzureIaasVM -v {vault} -g {rg}', checks=[
            self.check("length(@)", 2),
            self.check("length([?properties.friendlyName == '{vm1}'])", 1),
            self.check("length([?properties.friendlyName == '{vm2}'])", 1)])

    @ResourceGroupPreparer(location="southeastasia")
    @VaultPreparer()
    @PolicyPreparer(parameter_name='policy1')
    @PolicyPreparer(parameter_name='policy2', instant_rp_days="3")
    @VMPreparer(parameter_name='vm1')
    @VMPreparer(parameter_name='vm2')
    @ItemPreparer(vm_parameter_name='vm1')
    @ItemPreparer(vm_parameter_name='vm2')
    def test_backup_policy(self, resource_group, vault_name, policy1, policy2, vm1, vm2):

        self.kwargs.update({
            'policy1': policy1,
            'policy2': policy2,
            'policy3': self.create_random_name('clitest-policy', 24),
            'default': 'DefaultPolicy',
            'vault': vault_name,
            'vm1': vm1,
            'vm2': vm2,
            'policy5': self.create_random_name('clitest-policy5', 24),
        })

        self.cmd('backup vault backup-properties set -g {rg} -n {vault} --soft-delete-feature-state Disable')

        self.kwargs['policy1_json'] = self.cmd('backup policy show -g {rg} -v {vault} -n {policy1}', checks=[
            self.check('name', '{policy1}'),
            self.check('resourceGroup', '{rg}')
        ]).get_output_in_json()

        self.cmd('backup policy list -g {rg} -v {vault}', checks=[
            self.check("length(@)", 4),
            self.check("length([?name == '{default}'])", 1),
            self.check("length([?name == '{policy1}'])", 1),
            self.check("length([?name == '{policy2}'])", 1)
        ])

        self.cmd('backup policy list-associated-items -g {rg} -v {vault} -n {default}', checks=[
            self.check("length(@)", 2),
            self.check("length([?properties.friendlyName == '{}'])".format(vm1), 1),
            self.check("length([?properties.friendlyName == '{}'])".format(vm2), 1)
        ])

        self.kwargs['policy1_json']['name'] = self.kwargs['policy5']
        self.kwargs['backup-management-type'] = self.kwargs['policy1_json']['properties']['backupManagementType']
        self.kwargs['policy5_json'] = json.dumps(self.kwargs['policy1_json'])
        self.cmd("backup policy create --backup-management-type {backup-management-type} -g {rg} -v {vault} -n {policy5} --policy '{policy5_json}'")

        self.cmd('backup policy delete -g {rg} -v {vault} -n {policy5}')

        self.kwargs['policy1_json']['name'] = self.kwargs['policy3']
        if 'instantRpDetails' in self.kwargs['policy1_json']['properties']:
            self.kwargs['policy1_json']['properties']['instantRpDetails'] = {'azureBackupRgNamePrefix': 'RG_prefix', 'azureBackupRgNameSuffix': 'RG_suffix'}
        self.kwargs['policy1_json'] = json.dumps(self.kwargs['policy1_json'])

        self.cmd("backup policy set -g {rg} -v {vault} --policy '{policy1_json}'", checks=[
            self.check('name', '{policy3}'),
            self.check('resourceGroup', '{rg}')
        ])

        self.cmd('backup policy list -g {rg} -v {vault}', checks=[
            self.check("length(@)", 5),
            self.check("length([?name == '{default}'])", 1),
            self.check("length([?name == '{policy1}'])", 1),
            self.check("length([?name == '{policy2}'])", 1),
            self.check("length([?name == '{policy3}'])", 1)
        ])

        self.cmd('backup policy delete -g {rg} -v {vault} -n {policy3}')

        self.cmd('backup policy list -g {rg} -v {vault}', checks=[
            self.check("length(@)", 4),
            self.check("length([?name == '{default}'])", 1),
            self.check("length([?name == '{policy1}'])", 1),
            self.check("length([?name == '{policy2}'])", 1)
        ])

        self.kwargs['policy4_json'] = self.cmd('backup policy show -g {rg} -v {vault} -n {policy2}').get_output_in_json()
        self.assertEqual(self.kwargs['policy4_json']['properties']['instantRpRetentionRangeInDays'], 3)

    @ResourceGroupPreparer(location="southeastasia")
    @VaultPreparer()
    @VMPreparer(parameter_name='vm1')
    @VMPreparer(parameter_name='vm2')
    @ItemPreparer(vm_parameter_name='vm1')
    @ItemPreparer(vm_parameter_name='vm2')
    @PolicyPreparer()
    def test_backup_item(self, resource_group, vault_name, vm1, vm2, policy_name):

        self.kwargs.update({
            'vault': vault_name,
            'vm1': vm1,
            'vm2': vm2,
            'policy': policy_name,
            'default': 'DefaultPolicy'
        })
        self.kwargs['container1'] = self.cmd('backup container show --backup-management-type AzureIaasVM -n {vm1} -v {vault} -g {rg} --query properties.friendlyName').get_output_in_json()
        self.kwargs['container2'] = self.cmd('backup container show --backup-management-type AzureIaasVM -n {vm2} -v {vault} -g {rg} --query properties.friendlyName').get_output_in_json()

        item1_json = self.cmd('backup item show --backup-management-type AzureIaasVM --workload-type VM -g {rg} -v {vault} -c {container1} -n {vm1}', checks=[
            self.check('properties.friendlyName', '{vm1}'),
            self.check('properties.healthStatus', 'Passed'),
            self.check('properties.protectionState', 'IRPending'),
            self.check('properties.protectionStatus', 'Healthy'),
            self.check('resourceGroup', '{rg}')
        ]).get_output_in_json()

        self.assertIn(vault_name.lower(), item1_json['id'].lower())
        self.assertIn(vm1.lower(), item1_json['name'].lower())
        self.assertIn(vm1.lower(), item1_json['properties']['sourceResourceId'].lower())
        self.assertIn(vm1.lower(), item1_json['properties']['virtualMachineId'].lower())
        self.assertIn(self.kwargs['default'].lower(), item1_json['properties']['policyId'].lower())

        self.kwargs['container1_fullname'] = self.cmd('backup container show --backup-management-type AzureIaasVM -n {vm1} -v {vault} -g {rg} --query name').get_output_in_json()

        self.cmd('backup item show --workload-type VM -g {rg} -v {vault} -c {container1_fullname} -n {vm1}', checks=[
            self.check('properties.friendlyName', '{vm1}'),
            self.check('properties.healthStatus', 'Passed'),
            self.check('properties.protectionState', 'IRPending'),
            self.check('properties.protectionStatus', 'Healthy'),
            self.check('resourceGroup', '{rg}')
        ])

        self.kwargs['item1_fullname'] = item1_json['name']

        self.cmd('backup item show --backup-management-type AzureIaasVM --workload-type VM -g {rg} -v {vault} -c {container1_fullname} -n {item1_fullname}', checks=[
            self.check('properties.friendlyName', '{vm1}'),
            self.check('properties.healthStatus', 'Passed'),
            self.check('properties.protectionState', 'IRPending'),
            self.check('properties.protectionStatus', 'Healthy'),
            self.check('resourceGroup', '{rg}')
        ])

        self.cmd('backup item list --backup-management-type AzureIaasVM --workload-type VM -g {rg} -v {vault} -c {container1}', checks=[
            self.check("length(@)", 1),
            self.check("length([?properties.friendlyName == '{vm1}'])", 1)
        ])

        self.cmd('backup item list --backup-management-type AzureIaasVM --workload-type VM -g {rg} -v {vault} -c {container1_fullname}', checks=[
            self.check("length(@)", 1),
            self.check("length([?properties.friendlyName == '{vm1}'])", 1)
        ])

        self.cmd('backup item list --backup-management-type AzureIaasVM --workload-type VM -g {rg} -v {vault} -c {container2}', checks=[
            self.check("length(@)", 1),
            self.check("length([?properties.friendlyName == '{vm2}'])", 1)
        ])

        self.cmd('backup item list --backup-management-type AzureIaasVM --workload-type VM -g {rg} -v {vault}', checks=[
            self.check("length(@)", 2),
            self.check("length([?properties.friendlyName == '{vm1}'])", 1),
            self.check("length([?properties.friendlyName == '{vm2}'])", 1)
        ])

        self.cmd('backup item set-policy --backup-management-type AzureIaasVM --workload-type VM -g {rg} -v {vault} -c {container1} -n {vm1} -p {policy}', checks=[
            self.check("properties.entityFriendlyName", '{vm1}'),
            self.check("properties.operation", "ConfigureBackup"),
            self.check("properties.status", "Completed"),
            self.check("resourceGroup", '{rg}')
        ])

        item1_json = self.cmd('backup item show --backup-management-type AzureIaasVM --workload-type VM -g {rg} -v {vault} -c {container1} -n {vm1}').get_output_in_json()
        self.assertIn(policy_name.lower(), item1_json['properties']['policyId'].lower())

    @ResourceGroupPreparer(location="southeastasia")
    @VaultPreparer()
    @VMPreparer()
    @ItemPreparer()
    @RPPreparer()
    @RPPreparer()
    def test_backup_rp(self, resource_group, vault_name, vm_name):

        self.kwargs.update({
            'vault': vault_name,
            'vm': vm_name
        })

        rp_names = self.cmd('backup recoverypoint list --backup-management-type AzureIaasVM --workload-type VM -g {rg} -v {vault} -c {vm} -i {vm} --query [].name', checks=[
            self.check("length(@)", 2)
        ]).get_output_in_json()

        self.kwargs['rp1'] = rp_names[0]
        rp1_json = self.cmd('backup recoverypoint show --backup-management-type AzureIaasVM --workload-type VM -g {rg} -v {vault} -c {vm} -i {vm} -n {rp1}', checks=[
            self.check("name", '{rp1}'),
            self.check("resourceGroup", '{rg}')
        ]).get_output_in_json()
        self.assertIn(vault_name.lower(), rp1_json['id'].lower())
        self.assertIn(vm_name.lower(), rp1_json['id'].lower())

        self.kwargs['rp2'] = rp_names[1]
        rp2_json = self.cmd('backup recoverypoint show --backup-management-type AzureIaasVM --workload-type VM -g {rg} -v {vault} -c {vm} -i {vm} -n {rp2}', checks=[
            self.check("name", '{rp2}'),
            self.check("resourceGroup", '{rg}')
        ]).get_output_in_json()
        self.assertIn(vault_name.lower(), rp2_json['id'].lower())
        self.assertIn(vm_name.lower(), rp2_json['id'].lower())

    @ResourceGroupPreparer(location="southeastasia")
    @VaultPreparer(soft_delete=False)
    @VMPreparer()
    def test_backup_protection(self, resource_group, vault_name, vm_name):

        self.kwargs.update({
            'vault': vault_name,
            'vm': vm_name
        })

        self.kwargs['vm_id'] = self.cmd('vm show -g {rg} -n {vm} --query id').get_output_in_json()

        protection_check = self.cmd('backup protection check-vm --vm-id {vm_id}').output
        self.assertTrue(protection_check == '')

        self.cmd('backup protection enable-for-vm -g {rg} -v {vault} --vm {vm} -p DefaultPolicy', checks=[
            self.check("properties.entityFriendlyName", '{vm}'),
            self.check("properties.operation", "ConfigureBackup"),
            self.check("properties.status", "Completed"),
            self.check("resourceGroup", '{rg}')
        ])

        vault_id = self.cmd('backup vault show -g {rg} -n {vault} --query id').get_output_in_json()

        vault_id_check = self.cmd('backup protection check-vm --vm-id {vm_id}').get_output_in_json()
        self.assertIsNotNone(vault_id_check)
        self.assertTrue(vault_id.lower() == vault_id_check.lower())

        self.cmd('backup protection disable --backup-management-type AzureIaasVM --workload-type VM -g {rg} -v {vault} -c {vm} -i {vm} --yes', checks=[
            self.check("properties.entityFriendlyName", '{vm}'),
            self.check("properties.operation", "DisableBackup"),
            self.check("properties.status", "Completed"),
            self.check("resourceGroup", '{rg}')
        ])

        self.cmd('backup item show --backup-management-type AzureIaasVM --workload-type VM -g {rg} -v {vault} -c {vm} -n {vm}', checks=[
            self.check("properties.friendlyName", '{vm}'),
            self.check("properties.protectionState", "ProtectionStopped"),
            self.check("resourceGroup", '{rg}')
        ])

        self.cmd('backup protection disable --backup-management-type AzureIaasVM --workload-type VM -g {rg} -v {vault} -c {vm} -i {vm} --delete-backup-data true --yes', checks=[
            self.check("properties.entityFriendlyName", '{vm}'),
            self.check("properties.operation", "DeleteBackupData"),
            self.check("properties.status", "Completed"),
            self.check("resourceGroup", '{rg}')
        ])

        self.cmd('backup container list --backup-management-type AzureIaasVM -v {vault} -g {rg}',
                 checks=self.check("length(@)", 0))

        protection_check = self.cmd('backup protection check-vm --vm-id {vm_id}').output
        self.assertTrue(protection_check == '')

    @AllowLargeResponse()
    @ResourceGroupPreparer(location="southeastasia")
    @ResourceGroupPreparer(parameter_name="target_resource_group", location="southeastasia")
    @VaultPreparer()
    @VMPreparer()
    @ItemPreparer()
    @RPPreparer()
    @StorageAccountPreparer(location="southeastasia")
    def test_backup_restore(self, resource_group, target_resource_group, vault_name, vm_name, storage_account):

        self.kwargs.update({
            'vault': vault_name,
            'vm': vm_name,
            'target_rg': target_resource_group,
            'rg': resource_group
        })
        self.kwargs['rp'] = self.cmd('backup recoverypoint list --backup-management-type AzureIaasVM --workload-type VM -g {rg} -v {vault} -c {vm} -i {vm} --query [0].name').get_output_in_json()

        # Original Storage Account Restore Fails
        self.cmd('backup restore restore-disks -g {rg} -v {vault} -c {vm} -i {vm} -r {rp} --storage-account {sa} --restore-to-staging-storage-account false', expect_failure=True)

        # Trigger Restore
        trigger_restore_job_json = self.cmd('backup restore restore-disks -g {rg} -v {vault} -c {vm} -i {vm} -r {rp} -t {target_rg} --storage-account {sa} --restore-to-staging-storage-account', checks=[
            self.check("properties.entityFriendlyName", '{vm}'),
            self.check("properties.operation", "Restore"),
            self.check("properties.status", "InProgress"),
            self.check("resourceGroup", '{rg}')
        ]).get_output_in_json()
        self.kwargs['job'] = trigger_restore_job_json['name']
        self.cmd('backup job wait -g {rg} -v {vault} -n {job}')

        trigger_restore_job_details = self.cmd('backup job show -g {rg} -v {vault} -n {job}', checks=[
            self.check("properties.entityFriendlyName", '{vm}'),
            self.check("properties.operation", "Restore"),
            self.check("properties.status", "Completed"),
            self.check("resourceGroup", '{rg}')
        ]).get_output_in_json()

        property_bag = trigger_restore_job_details['properties']['extendedInfo']['propertyBag']
        self.assertEqual(property_bag['Target Storage Account Name'], storage_account)

        self.kwargs['container'] = property_bag['Config Blob Container Name']
        self.kwargs['blob'] = property_bag['Config Blob Name']

        self.cmd('storage blob exists --account-name {sa} -c {container} -n {blob}',
                 checks=self.check("exists", True))

        # Trigger Restore As unmanaged disks
        trigger_restore_job2_json = self.cmd('backup restore restore-disks -g {rg} -v {vault} -c {vm} -i {vm} -r {rp} --restore-as-unmanaged-disks --storage-account {sa}', checks=[
            self.check("properties.entityFriendlyName", '{vm}'),
            self.check("properties.operation", "Restore"),
            self.check("properties.status", "InProgress"),
            self.check("resourceGroup", '{rg}')
        ]).get_output_in_json()
        self.kwargs['job2'] = trigger_restore_job2_json['name']
        self.cmd('backup job wait -g {rg} -v {vault} -n {job2}')

        self.cmd('backup job show -g {rg} -v {vault} -n {job2}', checks=[
            self.check("properties.entityFriendlyName", '{vm}'),
            self.check("properties.operation", "Restore"),
            self.check("properties.status", "Completed"),
            self.check("resourceGroup", '{rg}')
        ])

    @ResourceGroupPreparer(location="southeastasia")
    @VaultPreparer()
    @VMPreparer()
    @ItemPreparer()
    @RPPreparer()
    @StorageAccountPreparer(location="southeastasia")
    def test_backup_job(self, resource_group, vault_name, vm_name, storage_account):

        self.kwargs.update({
            'vault': vault_name,
            'vm': vm_name
        })
        self.kwargs['rp'] = self.cmd('backup recoverypoint list --backup-management-type AzureIaasVM --workload-type VM -g {rg} -v {vault} -c {vm} -i {vm} --query [0].name').get_output_in_json()
        self.kwargs['job'] = self.cmd('backup restore restore-disks -g {rg} -v {vault} -c {vm} -i {vm} -r {rp} --storage-account {sa} --query name --restore-to-staging-storage-account').get_output_in_json()

        self.cmd('backup job show -g {rg} -v {vault} -n {job}', checks=[
            self.check("name", '{job}'),
            self.check("properties.entityFriendlyName", '{vm}'),
            self.check("properties.operation", "Restore"),
            self.check("properties.status", "InProgress"),
            self.check("resourceGroup", '{rg}')
        ])

        self.cmd('backup job list -g {rg} -v {vault}',
                 checks=self.check("length([?name == '{job}'])", 1))

        self.cmd('backup job list -g {rg} -v {vault} --status InProgress',
                 checks=self.check("length([?name == '{job}'])", 1))

        self.cmd('backup job list -g {rg} -v {vault} --operation Restore',
                 checks=self.check("length([?name == '{job}'])", 1))

        self.cmd('backup job list -g {rg} -v {vault} --operation Restore --status InProgress',
                 checks=self.check("length([?name == '{job}'])", 1))

        self.cmd('backup job stop -g {rg} -v {vault} -n {job}')

    @ResourceGroupPreparer()
    @VaultPreparer()
    @VMPreparer()
    @ItemPreparer()
    def test_backup_softdelete(self, resource_group, vault_name, vm_name):
        self.kwargs.update({
            'vault': vault_name,
            'vm': vm_name,
            'rg': resource_group
        })

        self.cmd('backup protection disable --backup-management-type AzureIaasVM --workload-type VM -g {rg} -v {vault} -c {vm} -i {vm} --delete-backup-data --yes', checks=[
            self.check("properties.entityFriendlyName", '{vm}'),
            self.check("properties.operation", "DeleteBackupData"),
            self.check("properties.status", "Completed"),
            self.check("resourceGroup", '{rg}')
        ])

        self.cmd('backup item show --backup-management-type AzureIaasVM --workload-type VM -g {rg} -v {vault} -c {vm} -n {vm}', checks=[
            self.check("properties.friendlyName", '{vm}'),
            self.check("properties.protectionState", "ProtectionStopped"),
            self.check("resourceGroup", '{rg}'),
            self.check("properties.isScheduledForDeferredDelete", True)
        ])

        self.cmd('backup protection undelete -g {rg} -v {vault} -c {vm} -i {vm} --workload-type VM --backup-management-type AzureIaasVM ', checks=[
            self.check("properties.entityFriendlyName", '{vm}'),
            self.check("properties.operation", "Undelete"),
            self.check("properties.status", "Completed"),
            self.check("resourceGroup", '{rg}')
        ])

        self.cmd('backup vault backup-properties set -g {rg} -n {vault} --soft-delete-feature-state Disable')

        self.cmd('backup item show --backup-management-type AzureIaasVM --workload-type VM -g {rg} -v {vault} -c {vm} -n {vm}', checks=[
            self.check("properties.friendlyName", '{vm}'),
            self.check("properties.protectionState", "ProtectionStopped"),
            self.check("resourceGroup", '{rg}'),
            self.check("properties.isScheduledForDeferredDelete", None)
        ])

    @ResourceGroupPreparer(location="southeastasia")
    @VaultPreparer()
    @VMPreparer()
    @StorageAccountPreparer(location="southeastasia")
    def test_backup_disk_exclusion(self, resource_group, vault_name, vm_name, storage_account):
        self.kwargs.update({
            'vault': vault_name,
            'vm': vm_name,
            'rg': resource_group,
            'sa': storage_account
        })

        self.cmd('backup vault backup-properties set -g {rg} -n {vault} --soft-delete-feature-state Disable')

        self.cmd('vm disk attach -g {rg} --vm-name {vm} --name mydisk1 --new --size-gb 10')
        self.cmd('vm disk attach -g {rg} --vm-name {vm} --name mydisk2 --new --size-gb 10')
        self.cmd('vm disk attach -g {rg} --vm-name {vm} --name mydisk3 --new --size-gb 10')

        self.cmd('backup protection enable-for-vm -g {rg} -v {vault} --vm {vm} -p DefaultPolicy --disk-list-setting include --diskslist 0 1', checks=[
            self.check("properties.entityFriendlyName", '{vm}'),
            self.check("properties.operation", "ConfigureBackup"),
            self.check("properties.status", "Completed"),
            self.check("resourceGroup", '{rg}')
        ])

        self.cmd('backup item show --backup-management-type AzureIaasVM --workload-type VM -g {rg} -v {vault} -c {vm} -n {vm}', checks=[
            self.check('properties.friendlyName', '{vm}'),
            self.check('properties.healthStatus', 'Passed'),
            self.check('properties.protectionState', 'IRPending'),
            self.check('properties.protectionStatus', 'Healthy'),
            self.check('resourceGroup', '{rg}'),
            self.check('properties.extendedProperties.diskExclusionProperties.isInclusionList', True),
        ])

        self.cmd('backup protection update-for-vm -g {rg} -v {vault} -c {vm} -i {vm} --exclude-all-data-disks', checks=[
            self.check("properties.entityFriendlyName", '{vm}'),
            self.check("properties.operation", "ConfigureBackup"),
            self.check("properties.status", "Completed"),
            self.check("resourceGroup", '{rg}')
        ])

        self.cmd('backup protection update-for-vm -g {rg} -v {vault} -c {vm} -i {vm} --disk-list-setting exclude --diskslist 1', checks=[
            self.check("properties.entityFriendlyName", '{vm}'),
            self.check("properties.operation", "ConfigureBackup"),
            self.check("properties.status", "Completed"),
            self.check("resourceGroup", '{rg}')
        ])

        self.cmd('backup item show --backup-management-type AzureIaasVM --workload-type VM -g {rg} -v {vault} -c {vm} -n {vm}', checks=[
            self.check('properties.friendlyName', '{vm}'),
            self.check('properties.healthStatus', 'Passed'),
            self.check('properties.protectionState', 'IRPending'),
            self.check('properties.protectionStatus', 'Healthy'),
            self.check('resourceGroup', '{rg}'),
            self.check('properties.extendedProperties.diskExclusionProperties.isInclusionList', False),
        ])

        self.kwargs['retain_date'] = (datetime.utcnow() + timedelta(days=30)).strftime('%d-%m-%Y')
        self.kwargs['job'] = self.cmd('backup protection backup-now -g {rg} -v {vault} -c {vm} -i {vm} --backup-management-type AzureIaasVM --workload-type VM --retain-until {retain_date} --query name').get_output_in_json()
        self.cmd('backup job wait -g {rg} -v {vault} -n {job}')

        self.kwargs['rp'] = self.cmd('backup recoverypoint list --backup-management-type AzureIaasVM --workload-type VM -g {rg} -v {vault} -c {vm} -i {vm} --query [0].name').get_output_in_json()

        trigger_restore_job_json = self.cmd('backup restore restore-disks -g {rg} -v {vault} -c {vm} -i {vm} -r {rp} -t {rg} --storage-account {sa} --restore-to-staging-storage-account --restore-only-osdisk', checks=[
            self.check("properties.entityFriendlyName", '{vm}'),
            self.check("properties.operation", "Restore"),
            self.check("properties.status", "InProgress"),
            self.check("resourceGroup", '{rg}')
        ]).get_output_in_json()
        self.kwargs['job'] = trigger_restore_job_json['name']
        self.cmd('backup job wait -g {rg} -v {vault} -n {job}')

        self.cmd('backup job show -g {rg} -v {vault} -n {job}', checks=[
            self.check("properties.entityFriendlyName", '{vm}'),
            self.check("properties.operation", "Restore"),
            self.check("properties.status", "Completed"),
            self.check("resourceGroup", '{rg}')
        ])

        trigger_restore_job_json = self.cmd('backup restore restore-disks -g {rg} -v {vault} -c {vm} -i {vm} -r {rp} -t {rg} --storage-account {sa} --restore-to-staging-storage-account --diskslist 0', checks=[
            self.check("properties.entityFriendlyName", '{vm}'),
            self.check("properties.operation", "Restore"),
            self.check("properties.status", "InProgress"),
            self.check("resourceGroup", '{rg}')
        ]).get_output_in_json()
        self.kwargs['job2'] = trigger_restore_job_json['name']
        self.cmd('backup job wait -g {rg} -v {vault} -n {job2}')

        self.cmd('backup job show -g {rg} -v {vault} -n {job2}', checks=[
            self.check("properties.entityFriendlyName", '{vm}'),
            self.check("properties.operation", "Restore"),
            self.check("properties.status", "Completed"),
            self.check("resourceGroup", '{rg}')
        ])



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import json

reserved_policy_list = [
  "36e07b214d59455886a2b76b",
  "5b5ec388a1b6480391640d13",
  "5d00856604b74b80927cca6e",
  "8805e4466db647d1beda40e2",
  "DataProtectionSecurityCenter",
  "SecurityCenterBuiltIn"
]


def main():
    print('Clean policy tool 0.1')

    cmd = 'az policy assignment list --query [].name'
    print(cmd)
    result = os.popen(cmd).read()
    for name in json.loads(result):
        if name not in reserved_policy_list:
            cmd = 'az policy assignment delete -n {}'.format(name)
            print(cmd)
            os.popen(cmd)


if __name__ == '__main__':
    main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import (ScenarioTest, JMESPathCheck, ResourceGroupPreparer, StorageAccountPreparer)
from azure.core.exceptions import HttpResponseError
from azure_devtools.scenario_tests import AllowLargeResponse


class StorageImmutabilityPolicy(ScenarioTest):
    @AllowLargeResponse()
    @ResourceGroupPreparer()
    @StorageAccountPreparer(kind="StorageV2")
    def test_immutability_policy(self, resource_group, storage_account):
        container_name = 'container1'
        self.cmd('storage container create --account-name {} -n {}'.format(storage_account, container_name))

        self.cmd('az storage container immutability-policy create --account-name {} -c {} -g {} --period 1 -w'.format(
            storage_account, container_name, resource_group), checks=[
                JMESPathCheck('immutabilityPeriodSinceCreationInDays', 1),
                JMESPathCheck('allowProtectedAppendWrites', True)
        ])

        policy_etag = self.cmd('az storage container immutability-policy show --account-name {} -c {} -g {}'.format(
            storage_account, container_name, resource_group), checks=[
                JMESPathCheck('immutabilityPeriodSinceCreationInDays', 1),
                JMESPathCheck('allowProtectedAppendWrites', True)
        ]).get_output_in_json().get('etag')

        self.cmd('az storage container immutability-policy delete --account-name {} -c {} -g {} --if-match {}'.format(
            storage_account, container_name, resource_group, repr(policy_etag)))

        self.cmd('az storage container immutability-policy show --account-name {} -c {} -g {}'.format(
            storage_account, container_name, resource_group))

        self.cmd('az storage container immutability-policy create --account-name {} -c {} -g {} --period 1 -w false'.format(
            storage_account, container_name, resource_group), checks=[
                JMESPathCheck('immutabilityPeriodSinceCreationInDays', 1),
                JMESPathCheck('allowProtectedAppendWrites', False)])

        policy_etag = self.cmd('az storage container immutability-policy show --account-name {} -c {} -g {}'.format(
            storage_account, container_name, resource_group), checks=[
                JMESPathCheck('immutabilityPeriodSinceCreationInDays', 1),
                JMESPathCheck('allowProtectedAppendWrites', False)
        ]).get_output_in_json().get('etag')

        self.cmd('az storage container immutability-policy lock --account-name {} -c {} -g {} --if-match {}'.format(
            storage_account, container_name, resource_group, repr(policy_etag)))

        policy_etag = self.cmd('az storage container immutability-policy show --account-name {} -c {} -g {}'.format(
            storage_account, container_name, resource_group), checks=[
                JMESPathCheck('immutabilityPeriodSinceCreationInDays', 1)]).get_output_in_json().get('etag')

        # cannot delete locked policy
        with self.assertRaises(HttpResponseError):
            self.cmd('az storage container immutability-policy delete --account-name {} -c {} -g {} '
                     '--if-match {}'.format(storage_account, container_name, resource_group, repr(policy_etag)))

        self.cmd('az storage container immutability-policy extend --period 2 '
                 '--account-name {} -c {} -g {} --if-match {}'.format(
                     storage_account, container_name, resource_group, repr(policy_etag)), checks=[
                         JMESPathCheck('immutabilityPeriodSinceCreationInDays', 2)])

        self.cmd('az storage container delete --account-name {} -n {} --bypass-immutability-policy'.format(
            storage_account, container_name))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

"""Verify the document source map. """

from __future__ import print_function

import os
import sys
import json

from ..utilities.path import (get_repo_root, get_command_modules_paths)
from ..utilities.display import print_heading
from ..utilities.const import COMMAND_MODULE_PREFIX

REPO_ROOT = get_repo_root()
DOC_MAP_NAME = 'doc_source_map.json'
HELP_FILE_NAME = '_help.py'
DOC_SOURCE_MAP_PATH = os.path.join('doc', 'sphinx', 'azhelpgen', DOC_MAP_NAME)


def _get_help_files_in_map(map_path):
    with open(map_path) as json_file:
        json_data = json.load(json_file)
        return list(json_data.values())


def _map_help_files_not_found(help_files_in_map):
    none_existent_files = []
    for f in help_files_in_map:
        if not os.path.isfile(os.path.join(REPO_ROOT, f)):
            none_existent_files.append(f)
    return none_existent_files


def _help_files_not_in_map(help_files_in_map):
    found_files = []
    not_in_map = []
    for name, path in get_command_modules_paths():
        name.replace(COMMAND_MODULE_PREFIX, '')
        help_file = os.path.join(path, 'azure', 'cli', 'command_modules', name, HELP_FILE_NAME)
        if os.path.isfile(help_file):
            found_files.append(help_file)
    for f in found_files:
        f_path = f.replace(REPO_ROOT + '/', '')
        if f_path not in help_files_in_map:
            not_in_map.append(f_path)
    return not_in_map


def verify_doc_source_map(*args):
    map_path = os.path.join(REPO_ROOT, DOC_SOURCE_MAP_PATH)
    help_files_in_map = _get_help_files_in_map(map_path)
    help_files_not_found = _map_help_files_not_found(help_files_in_map)
    hep_files_to_add_to_map = _help_files_not_in_map(help_files_in_map)

    if help_files_not_found or hep_files_to_add_to_map:
        print_heading('Errors whilst verifying {}!'.format(DOC_MAP_NAME))
        if help_files_not_found:
            print('\nThe following files are in {} but do not exist:'.format(DOC_MAP_NAME), file=sys.stderr)
            print('\n'.join(help_files_not_found), file=sys.stderr)
        if hep_files_to_add_to_map:
            print('\nThe following files should be added to {}:'.format(DOC_MAP_NAME), file=sys.stderr)
            print('\n'.join(hep_files_to_add_to_map), file=sys.stderr)
        sys.exit(1)
    else:
        print('Verified {} successfully.'.format(DOC_MAP_NAME), file=sys.stderr)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest
from knack import CLI

from azure.cli.core._config import GLOBAL_CONFIG_DIR, ENV_VAR_PREFIX
from azure.cli.core.cloud import get_active_cloud

from azure.cli.command_modules.storage.storage_url_helpers import StorageResourceIdentifier


class MockCLI(CLI):
    def __init__(self):
        super(MockCLI, self).__init__(cli_name='mock_cli', config_dir=GLOBAL_CONFIG_DIR,
                                      config_env_var_prefix=ENV_VAR_PREFIX)
        self.cloud = get_active_cloud(self)


class TestStorageUrlHelpers(unittest.TestCase):
    def setUp(self):
        self.cli = MockCLI()

    def test_is_url(self):
        def _check_is_url(moniker, expectation):
            assert StorageResourceIdentifier(self.cli.cloud, moniker).is_url() == expectation

        _check_is_url('sample', False)
        _check_is_url('http://test.blob.core.windows.net/cont', True)
        _check_is_url('https://test.blob.core.windows.net/cont', True)
        _check_is_url('https://test.file.core.windows.net/cont', True)

    def test_container_parsing(self):
        def _test(moniker, expected_container=None, expected_blob=None, expected_snapshot=None):
            i = StorageResourceIdentifier(self.cli.cloud, moniker)

            if expected_container is not None:
                assert i.container == expected_container
                assert i.share is None
                assert i.directory is None
                assert i.filename is None
            else:
                assert i.container is None

            assert i.blob == expected_blob
            assert i.snapshot == expected_snapshot

        _test('sample')
        _test('https://momba.file.core.windows.net/snake')
        _test('http://momba.else.core.windows.net/snake')
        _test('http://momba.blob.core.windows.net/snake', 'snake')
        _test('https://momba.blob.core.windows.net/snake', 'snake')
        _test('https://momba.blob.core.windows.net/snake/blob', 'snake', 'blob')
        _test('https://momba.blob.core.windows.net/snake/blob/blob2', 'snake', 'blob/blob2')
        _test('https://momba.blob.core.windows.net/snake/blob?some=thing', 'snake', 'blob')

    def test_share_parsing(self):
        def _test(moniker, expected_share=None, expected_dir=None, expected_file=None):
            i = StorageResourceIdentifier(self.cli.cloud, moniker)

            if expected_share is not None:
                assert i.container is None
                assert i.blob is None
                assert i.snapshot is None
                assert i.share == expected_share
            else:
                assert i.share is None

            assert i.directory == expected_dir
            assert i.filename == expected_file

        _test('sample')
        _test('https://momba.blob.core.windows.net/snake')
        _test('http://momba.blob.core.windows.net/snake')
        _test('http://momba.else.core.windows.net/snake')
        _test('http://momba.file.core.windows.net/snake/d/f.txt', 'snake', 'd', 'f.txt')
        _test('http://momba.file.core.windows.net/snake/f.txt', 'snake', '', 'f.txt')
        _test('http://momba.file.core.windows.net/snake/d/e/f.txt', 'snake', 'd/e', 'f.txt')
        _test('http://momba.file.core.windows.net/snake/d/e/f.txt?s=t', 'snake', 'd/e', 'f.txt')

    def test_account_name(self):
        def _test(moniker, expected_account=None):
            i = StorageResourceIdentifier(self.cli.cloud, moniker)
            assert i.account_name == expected_account

        _test('sample')
        _test('https://momba.else.core.windows.net/snake')
        _test('https://momba.blob.core.windows.net/snake', 'momba')
        _test('http://momba.file.core.windows.net/snake', 'momba')
        _test('http://momba.file.core.windows.net/snake/d/e/f.txt?s=t', 'momba')

    def test_default_value(self):
        i = StorageResourceIdentifier(self.cli.cloud, '')
        assert not i.is_url()
        assert not i.is_valid
        assert i.account_name is None
        assert i.container is None
        assert i.blob is None
        assert i.share is None
        assert i.directory is None
        assert i.filename is None

    def test_get_sas_token(self):
        def _test(moniker, expected_sas=None):
            i = StorageResourceIdentifier(self.cli.cloud, moniker)
            assert i.sas_token == expected_sas

        _test('https://momba.blob.core.windows.net/blob?sv=2015-04-05&ss=bfqt&srt=sco&sp=rwdlacup&se='
              '2016-12-05T23:02:02Z&st=2016-12-05T15:02:02Z&spr=https&sig=e0xYWg%2F142F5uUsPBflsUVQqL'
              '33Pr0v3Fs5VIjsUL6A%3D',
              'sv=2015-04-05&ss=bfqt&srt=sco&sp=rwdlacup&se=2016-12-05T23:02:02Z&st=2016-12-05T15:02:'
              '02Z&spr=https&sig=e0xYWg%2F142F5uUsPBflsUVQqL33Pr0v3Fs5VIjsUL6A%3D')

        _test('https://momba.blob.core.windows.net/blob?sv=2015-04-05&ss=bfqt&srt=sco&sp=rwdlacup&se='
              '2016-12-05T23:02:02Z&st=2016-12-05T15:02:02Z&spr=https&sig=e0xYWg%2F142F5uUsPBflsUVQqL'
              '33Pr0v3Fs5VIjsUL6A%3D&snapshot=2016-12-05T23:12:03.1181304Z',
              'sv=2015-04-05&ss=bfqt&srt=sco&sp=rwdlacup&se=2016-12-05T23:02:02Z&st=2016-12-05T15:02:'
              '02Z&spr=https&sig=e0xYWg%2F142F5uUsPBflsUVQqL33Pr0v3Fs5VIjsUL6A%3D')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import json
import os
from azure.synapse.artifacts.models import (LinkedService, Dataset, PipelineResource, RunFilterParameters,
                                            Trigger, DataFlow, BigDataPoolReference, NotebookSessionProperties,
                                            Notebook)
from azure.cli.core.util import sdk_no_wait, CLIError
from .sparkpool import get_spark_pool
from .workspace import get_resource_group_by_workspace_name
from .._client_factory import (cf_synapse_linked_service, cf_synapse_dataset, cf_synapse_pipeline,
                               cf_synapse_pipeline_run, cf_synapse_trigger, cf_synapse_trigger_run,
                               cf_synapse_data_flow, cf_synapse_notebook, cf_synapse_client_bigdatapool_factory,
                               cf_synapse_client_workspace_factory)
from ..constant import EXECUTOR_SIZE, SPARK_SERVICE_ENDPOINT_API_VERSION


# Linked Service
def list_linked_service(cmd, workspace_name):
    client = cf_synapse_linked_service(cmd.cli_ctx, workspace_name)
    return client.get_linked_services_by_workspace()


def get_linked_service(cmd, workspace_name, linked_service_name):
    client = cf_synapse_linked_service(cmd.cli_ctx, workspace_name)
    return client.get_linked_service(linked_service_name)


def create_or_update_linked_service(cmd, workspace_name, linked_service_name, definition_file, no_wait=False):
    client = cf_synapse_linked_service(cmd.cli_ctx, workspace_name)
    properties = LinkedService.from_dict(definition_file['properties'])
    return sdk_no_wait(no_wait, client.begin_create_or_update_linked_service,
                       linked_service_name, properties, polling=True)


def delete_linked_service(cmd, workspace_name, linked_service_name, no_wait=False):
    client = cf_synapse_linked_service(cmd.cli_ctx, workspace_name)
    return sdk_no_wait(no_wait, client.begin_delete_linked_service, linked_service_name, polling=True)


# Dataset
def list_datasets(cmd, workspace_name):
    client = cf_synapse_dataset(cmd.cli_ctx, workspace_name)
    return client.get_datasets_by_workspace()


def get_dataset(cmd, workspace_name, dataset_name):
    client = cf_synapse_dataset(cmd.cli_ctx, workspace_name)
    return client.get_dataset(dataset_name)


def create_or_update_dataset(cmd, workspace_name, dataset_name, definition_file, no_wait=False):
    client = cf_synapse_dataset(cmd.cli_ctx, workspace_name)
    properties = Dataset.from_dict(definition_file['properties'])
    return sdk_no_wait(no_wait, client.begin_create_or_update_dataset,
                       dataset_name, properties, polling=True)


def delete_dataset(cmd, workspace_name, dataset_name, no_wait=False):
    client = cf_synapse_dataset(cmd.cli_ctx, workspace_name)
    return sdk_no_wait(no_wait, client.begin_delete_dataset, dataset_name, polling=True)


# Pipeline
def list_pipelines(cmd, workspace_name):
    client = cf_synapse_pipeline(cmd.cli_ctx, workspace_name)
    return client.get_pipelines_by_workspace()


def get_pipeline(cmd, workspace_name, pipeline_name):
    client = cf_synapse_pipeline(cmd.cli_ctx, workspace_name)
    return client.get_pipeline(pipeline_name)


def create_or_update_pipeline(cmd, workspace_name, pipeline_name, definition_file, no_wait=False):
    client = cf_synapse_pipeline(cmd.cli_ctx, workspace_name)
    properties = PipelineResource.from_dict(definition_file['properties'])
    return sdk_no_wait(no_wait, client.begin_create_or_update_pipeline,
                       pipeline_name, properties, polling=True)


def delete_pipeline(cmd, workspace_name, pipeline_name, no_wait=False):
    client = cf_synapse_pipeline(cmd.cli_ctx, workspace_name)
    return sdk_no_wait(no_wait, client.begin_delete_pipeline, pipeline_name, polling=True)


def create_pipeline_run(cmd, workspace_name, pipeline_name, reference_pipeline_run_id=None, is_recovery=None,
                        start_activity_name=None, parameters=None):
    client = cf_synapse_pipeline(cmd.cli_ctx, workspace_name)
    return client.create_pipeline_run(pipeline_name, reference_pipeline_run_id, is_recovery,
                                      start_activity_name, parameters)


# Pipline run
def query_pipeline_runs_by_workspace(cmd, workspace_name, last_updated_after, last_updated_before,
                                     continuation_token=None, filters=None, order_by=None):
    client = cf_synapse_pipeline_run(cmd.cli_ctx, workspace_name)
    return client.query_pipeline_runs_by_workspace(RunFilterParameters(last_updated_after=last_updated_after,
                                                                       last_updated_before=last_updated_before,
                                                                       continuation_token=continuation_token,
                                                                       filters=filters, order_by=order_by))


def get_pipeline_run(cmd, workspace_name, run_id):
    client = cf_synapse_pipeline_run(cmd.cli_ctx, workspace_name)
    return client.get_pipeline_run(run_id)


def query_activity_runs(cmd, workspace_name, pipeline_name, run_id, last_updated_after, last_updated_before,
                        continuation_token=None, filters=None, order_by=None):
    client = cf_synapse_pipeline_run(cmd.cli_ctx, workspace_name)
    return client.query_activity_runs(pipeline_name, run_id,
                                      RunFilterParameters(last_updated_after=last_updated_after,
                                                          last_updated_before=last_updated_before,
                                                          continuation_token=continuation_token,
                                                          filters=filters, order_by=order_by))


def cancel_pipeline_run(cmd, workspace_name, run_id, is_recursive=None):
    client = cf_synapse_pipeline_run(cmd.cli_ctx, workspace_name)
    return client.cancel_pipeline_run(run_id, is_recursive)


# Trigger
def list_triggers(cmd, workspace_name):
    client = cf_synapse_trigger(cmd.cli_ctx, workspace_name)
    return client.get_triggers_by_workspace()


def get_trigger(cmd, workspace_name, trigger_name):
    client = cf_synapse_trigger(cmd.cli_ctx, workspace_name)
    return client.get_trigger(trigger_name)


def create_or_update_trigger(cmd, workspace_name, trigger_name, definition_file, no_wait=False):
    client = cf_synapse_trigger(cmd.cli_ctx, workspace_name)
    properties = Trigger.from_dict(definition_file['properties'])
    return sdk_no_wait(no_wait, client.begin_create_or_update_trigger,
                       trigger_name, properties, polling=True)


def delete_trigger(cmd, workspace_name, trigger_name, no_wait=False):
    client = cf_synapse_trigger(cmd.cli_ctx, workspace_name)
    return sdk_no_wait(no_wait, client.begin_delete_trigger, trigger_name, polling=True)


def subscribe_trigger_to_events(cmd, workspace_name, trigger_name, no_wait=False):
    client = cf_synapse_trigger(cmd.cli_ctx, workspace_name)
    return sdk_no_wait(no_wait, client.begin_subscribe_trigger_to_events, trigger_name, polling=True)


def get_event_subscription_status(cmd, workspace_name, trigger_name):
    client = cf_synapse_trigger(cmd.cli_ctx, workspace_name)
    return client.get_event_subscription_status(trigger_name)


def unsubscribe_trigger_from_events(cmd, workspace_name, trigger_name, no_wait=False):
    client = cf_synapse_trigger(cmd.cli_ctx, workspace_name)
    return sdk_no_wait(no_wait, client.begin_unsubscribe_trigger_from_events, trigger_name, polling=True)


def start_trigger(cmd, workspace_name, trigger_name, no_wait=False):
    client = cf_synapse_trigger(cmd.cli_ctx, workspace_name)
    return sdk_no_wait(no_wait, client.begin_start_trigger, trigger_name, polling=True)


def stop_trigger(cmd, workspace_name, trigger_name, no_wait=False):
    client = cf_synapse_trigger(cmd.cli_ctx, workspace_name)
    return sdk_no_wait(no_wait, client.begin_stop_trigger, trigger_name, polling=True)


# Trigger run
def rerun_trigger(cmd, workspace_name, trigger_name, run_id):
    client = cf_synapse_trigger_run(cmd.cli_ctx, workspace_name)
    return client.rerun_trigger_instance(trigger_name, run_id)


def query_trigger_runs_by_workspace(cmd, workspace_name, last_updated_after, last_updated_before,
                                    continuation_token=None, filters=None, order_by=None):
    client = cf_synapse_trigger_run(cmd.cli_ctx, workspace_name)
    return client.query_trigger_runs_by_workspace(RunFilterParameters(last_updated_after=last_updated_after,
                                                                      last_updated_before=last_updated_before,
                                                                      continuation_token=continuation_token,
                                                                      filters=filters, order_by=order_by))


# Data flow
def list_data_flows(cmd, workspace_name):
    client = cf_synapse_data_flow(cmd.cli_ctx, workspace_name)
    return client.get_data_flows_by_workspace()


def get_data_flow(cmd, workspace_name, data_flow_name):
    client = cf_synapse_data_flow(cmd.cli_ctx, workspace_name)
    return client.get_data_flow(data_flow_name)


def create_or_update_data_flow(cmd, workspace_name, data_flow_name, definition_file, no_wait=False):
    client = cf_synapse_data_flow(cmd.cli_ctx, workspace_name)
    properties = DataFlow.from_dict(definition_file['properties'])
    return sdk_no_wait(no_wait, client.begin_create_or_update_data_flow,
                       data_flow_name, properties, polling=True)


def delete_data_flow(cmd, workspace_name, data_flow_name, no_wait=False):
    client = cf_synapse_data_flow(cmd.cli_ctx, workspace_name)
    return sdk_no_wait(no_wait, client.begin_delete_data_flow, data_flow_name, polling=True)


# Notebook
def create_or_update_notebook(cmd, workspace_name, definition_file, notebook_name, spark_pool_name=None,
                              executor_size="Small", executor_count=2, no_wait=False):
    client = cf_synapse_notebook(cmd.cli_ctx, workspace_name)
    if spark_pool_name is not None:
        endpoint = '{}{}{}'.format("https://", workspace_name, cmd.cli_ctx.cloud.suffixes.synapse_analytics_endpoint)
        resource_group_name = get_resource_group_by_workspace_name(cmd,
                                                                   cf_synapse_client_workspace_factory(cmd.cli_ctx),
                                                                   workspace_name)
        spark_pool_info = get_spark_pool(cmd, cf_synapse_client_bigdatapool_factory(cmd.cli_ctx), resource_group_name,
                                         workspace_name, spark_pool_name)
        metadata = definition_file['metadata']
        options = {}
        options['auth'] = {'type': 'AAD',
                           'authResource': '{}{}'.format("https://",
                                                         cmd.cli_ctx.cloud.suffixes.synapse_analytics_endpoint[1:])}
        options['cores'] = EXECUTOR_SIZE[executor_size]['Cores']
        options['memory'] = EXECUTOR_SIZE[executor_size]['Memory']
        options['nodeCount'] = executor_count
        options['endpoint'] = '{}{}{}{}{}'.format(endpoint, '/livyApi/versions/', SPARK_SERVICE_ENDPOINT_API_VERSION,
                                                  '/sparkPools/', spark_pool_name)
        options['extraHeader'] = {}
        options['id'] = spark_pool_info.id
        options['name'] = spark_pool_name
        options['sparkVersion'] = spark_pool_info.spark_version
        options['type'] = 'Spark'
        metadata['a365ComputeOptions'] = options

        definition_file['bigDataPool'] = BigDataPoolReference(type='BigDataPoolReference',
                                                              reference_name=spark_pool_name)
        definition_file['sessionProperties'] = NotebookSessionProperties(driver_memory=options['memory'],
                                                                         driver_cores=options['cores'],
                                                                         executor_memory=options['memory'],
                                                                         executor_cores=options['cores'],
                                                                         num_executors=executor_count)
    properties = Notebook.from_dict(definition_file)
    return sdk_no_wait(no_wait, client.begin_create_or_update_notebook,
                       notebook_name, properties, polling=True)


def list_notebooks(cmd, workspace_name):
    client = cf_synapse_notebook(cmd.cli_ctx, workspace_name)
    return client.get_notebooks_by_workspace()


def get_notebook(cmd, workspace_name, notebook_name):
    client = cf_synapse_notebook(cmd.cli_ctx, workspace_name)
    return client.get_notebook(notebook_name)


def export_notebook(cmd, workspace_name, output_folder, notebook_name=None):
    def write_to_file(notebook, path):
        try:
            with open(path, 'w') as f:
                json.dump(notebook.properties.as_dict(), f, indent=4)
        except IOError:
            raise CLIError('Unable to export to file: {}'.format(path))

    client = cf_synapse_notebook(cmd.cli_ctx, workspace_name)
    if notebook_name is not None:
        notebook = client.get_notebook(notebook_name)
        path = os.path.join(output_folder, notebook.name + '.ipynb')
        print(notebook.properties.as_dict())
        write_to_file(notebook, path)
    else:
        notebooks = client.get_notebooks_by_workspace()
        for notebook in notebooks:
            path = os.path.join(output_folder, notebook.name + '.ipynb')
            print(notebook.properties.as_dict())
            write_to_file(notebook, path)


def delete_notebook(cmd, workspace_name, notebook_name, no_wait=False):
    client = cf_synapse_notebook(cmd.cli_ctx, workspace_name)
    return sdk_no_wait(no_wait, client.begin_delete_notebook, notebook_name, polling=True)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer
LOCATION = "eastus2euap"


class AzureNetAppFilesBackupPolicyServiceScenarioTest(ScenarioTest):
    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_backup_policy_')
    def test_create_delete_backup_policies(self):
        # create account
        account_name = self.create_random_name(prefix='cli-acc-', length=24)
        self.cmd("az netappfiles account create -g {rg} -a '%s' -l %s" % (account_name, LOCATION)).get_output_in_json()

        # create backup policy using long parameter names and validate result
        backup_policy_name = self.create_random_name(prefix='cli-ba-pol-', length=16)
        daily_backups_to_keep = 1
        weekly_backups_to_keep = 2
        monthly_backups_to_keep = 3
        enabled = True
        tags = "Tag1=Value1 Tag2=Value2"
        backup_policy = self.cmd("az netappfiles account backup-policy create -g {rg} -a %s --backup-policy-name %s "
                                 "--location %s --daily-backups %s --weekly-backups %s "
                                 "--monthly-backups %s --enabled %s --tags %s" %
                                 (account_name, backup_policy_name, LOCATION, daily_backups_to_keep,
                                  weekly_backups_to_keep, monthly_backups_to_keep, enabled, tags)).get_output_in_json()
        assert backup_policy['name'] == account_name + "/" + backup_policy_name
        assert backup_policy['dailyBackupsToKeep'] == daily_backups_to_keep
        assert backup_policy['weeklyBackupsToKeep'] == weekly_backups_to_keep
        assert backup_policy['monthlyBackupsToKeep'] == monthly_backups_to_keep
        assert backup_policy['enabled'] == enabled
        assert backup_policy['tags']['Tag1'] == 'Value1'
        assert backup_policy['tags']['Tag2'] == 'Value2'

        # validate backup policy exist
        backup_policy_list = self.cmd("az netappfiles account backup-policy list -g {rg} -a '%s'" %
                                      account_name).get_output_in_json()
        assert len(backup_policy_list) == 1

        # delete backup policy
        self.cmd("az netappfiles account backup-policy delete -g {rg} -a %s --backup-policy-name %s" %
                 (account_name, backup_policy_name))

        # create backup policy using short parameter names and validate result
        backup_policy = self.cmd("az netappfiles account backup-policy create -g {rg} -a %s -b %s "
                                 "-l %s -d %s -w %s -m %s -e %s --tags %s" %
                                 (account_name, backup_policy_name, LOCATION, daily_backups_to_keep,
                                  weekly_backups_to_keep, monthly_backups_to_keep, enabled, tags)).get_output_in_json()
        assert backup_policy['name'] == account_name + "/" + backup_policy_name
        assert backup_policy['dailyBackupsToKeep'] == daily_backups_to_keep
        assert backup_policy['weeklyBackupsToKeep'] == weekly_backups_to_keep
        assert backup_policy['monthlyBackupsToKeep'] == monthly_backups_to_keep
        assert backup_policy['enabled'] == enabled
        assert backup_policy['tags']['Tag1'] == 'Value1'
        assert backup_policy['tags']['Tag2'] == 'Value2'

        # delete backup policy
        self.cmd("az netappfiles account backup-policy delete -g {rg} -a %s --backup-policy-name %s" %
                 (account_name, backup_policy_name))

        # validate backup policy doesn't exist
        backup_policy_list = self.cmd("az netappfiles account backup-policy list -g {rg} -a '%s'" %
                                      account_name).get_output_in_json()
        assert len(backup_policy_list) == 0

    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_backup_policy_')
    def test_list_backup_policy(self):
        # create account
        account_name = self.create_random_name(prefix='cli-acc-', length=24)
        self.cmd("az netappfiles account create -g {rg} -a '%s' -l %s" % (account_name, LOCATION)).get_output_in_json()

        # create 3 backup policies
        backup_policies = [self.create_random_name(prefix='cli', length=16),
                           self.create_random_name(prefix='cli', length=16),
                           self.create_random_name(prefix='cli', length=16)]
        for backup_policy_name in backup_policies:
            self.cmd("az netappfiles account backup-policy create -g {rg} -a %s --backup-policy-name %s -l %s "
                     "--daily-backups 1" % (account_name, backup_policy_name, LOCATION))

        # validate that both backup policies exist
        backup_policy_list = self.cmd("az netappfiles account backup-policy list -g {rg} -a '%s'" %
                                      account_name).get_output_in_json()
        assert len(backup_policy_list) == 3

        # delete all backup policies
        for backup_policy_name in backup_policies:
            self.cmd("az netappfiles account backup-policy delete -g {rg} -a %s --backup-policy-name %s" %
                     (account_name, backup_policy_name))

        # validate that no backup policies exist
        backup_policy_list = self.cmd("az netappfiles account backup-policy list -g {rg} -a '%s'" %
                                      account_name).get_output_in_json()
        assert len(backup_policy_list) == 0

    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_backup_policy_')
    def test_get_backup_policy_by_name(self):
        # create account
        account_name = self.create_random_name(prefix='cli-acc-', length=24)
        self.cmd("az netappfiles account create -g {rg} -a '%s' -l %s" % (account_name, LOCATION)).get_output_in_json()

        # create backup policy
        backup_policy_name = self.create_random_name(prefix='cli-sn-pol-', length=16)
        self.cmd("az netappfiles account backup-policy create -g {rg} -a %s --backup-policy-name %s -l %s "
                 "--daily-backups 1" % (account_name, backup_policy_name, LOCATION)).get_output_in_json()

        # get backup policy by name and validate
        backup_policy = self.cmd("az netappfiles account backup-policy show -g {rg} -a %s --backup-policy-name %s" %
                                 (account_name, backup_policy_name)).get_output_in_json()
        assert backup_policy['name'] == account_name + '/' + backup_policy_name
        assert backup_policy['dailyBackupsToKeep'] == 1

        # get backup policy by resource id and validate
        backup_policy_from_id = self.cmd("az netappfiles account backup-policy show --ids %s" %
                                         backup_policy['id']).get_output_in_json()
        assert backup_policy_from_id['name'] == account_name + '/' + backup_policy_name
        assert backup_policy['dailyBackupsToKeep'] == 1

    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_backup_policy_')
    def test_update_backup_policy(self):
        # create account
        account_name = self.create_random_name(prefix='cli-acc-', length=24)
        self.cmd("az netappfiles account create -g {rg} -a '%s' -l %s" % (account_name, LOCATION)).get_output_in_json()

        # create backup policy
        backup_policy_name = self.create_random_name(prefix='cli-sn-pol-', length=16)
        daily_backups_to_keep = 1
        weekly_backups_to_keep = 2
        monthly_backups_to_keep = 3
        enabled = True
        tags = "Tag1=Value1"
        self.cmd("az netappfiles account backup-policy create -g {rg} -a %s --backup-policy-name %s "
                 "-l %s -d %s -w %s -m %s -e %s --tags %s" %
                 (account_name, backup_policy_name, LOCATION, daily_backups_to_keep,
                  weekly_backups_to_keep, monthly_backups_to_keep, enabled, tags)).get_output_in_json()

        # update backup policy
        daily_backups_to_keep = 4
        weekly_backups_to_keep = 5
        monthly_backups_to_keep = 6
        enabled = False
        tags = "Tag1=Value2"
        self.cmd("az netappfiles account backup-policy update -g {rg} -a %s --backup-policy-name %s -l %s -d %s -w %s "
                 "-m %s -e %s --tags %s" % (account_name, backup_policy_name, LOCATION, daily_backups_to_keep,
                                            weekly_backups_to_keep, monthly_backups_to_keep, enabled, tags)).get_output_in_json()

        # get updated backup policy and validate update
        backup_policy = self.cmd("az netappfiles account backup-policy show -g {rg} -a %s --backup-policy-name %s" %
                                 (account_name, backup_policy_name)).get_output_in_json()
        assert backup_policy['dailyBackupsToKeep'] == daily_backups_to_keep
        assert backup_policy['weeklyBackupsToKeep'] == weekly_backups_to_keep
        assert backup_policy['monthlyBackupsToKeep'] == monthly_backups_to_keep
        assert backup_policy['enabled'] == enabled
        assert backup_policy['tags']['Tag1'] == 'Value2'



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from __future__ import print_function
import os
import json
import os
import platform
import tempfile
import time
import fileinput

from azure.cli.testsdk import (
    ScenarioTest,
    ResourceGroupPreparer,
    StorageAccountPreparer)

from msrestazure.azure_exceptions import CloudError
from azure_devtools.scenario_tests import AllowLargeResponse

name_prefix = 'cliadm'
resource_location = 'centralus'
TEST_DIR = os.path.abspath(os.path.join(os.path.abspath(__file__), '..'))
curr_dir = os.path.dirname(os.path.realpath(__file__))

create_rollout_template = os.path.join(curr_dir, "createrollout.json").replace('\\', '\\\\')
failure_create_rollout_template = os.path.join(curr_dir, "createrollout_failurerollout.json").replace('\\', '\\\\')
healthcheck_file_path = os.path.join(curr_dir, "healthcheck_step.json").replace('\\', '\\\\')

parameters_file_name = "storage.parameters.json"
invalid_parameters_file_name = "storage_invalid.parameters.json"
template_file_name = "storage.template.json"
params_copy_file_name = "storage.copy.parameters.json"
template_copy_file_name = "storage.copy.template.json"
path_to_tests = ".\\azure\\cli\\command_modules\\deploymentmanager\\tests\\latest\\"
artifact_root = "artifactroot"

parametersArtifactSourceRelativePath = os.path.join(curr_dir, artifact_root, parameters_file_name)
templateArtifactSourceRelativePath = os.path.join(curr_dir, artifact_root, template_file_name)
invalidParametersArtifactSourceRelativePath = os.path.join(curr_dir, artifact_root, invalid_parameters_file_name)
parametersCopyArtifactSourceRelativePath = os.path.join(curr_dir, artifact_root, params_copy_file_name)
templateCopyArtifactSourceRelativePath = os.path.join(curr_dir, artifact_root, template_copy_file_name)


class DeploymentManagerTests(ScenarioTest):

    @ResourceGroupPreparer(name_prefix=name_prefix, random_name_length=12, location=resource_location)
    @StorageAccountPreparer(name_prefix=name_prefix, location=resource_location)
    @AllowLargeResponse()
    def test_deploymentmanager_scenario(self, resource_group, storage_account):

        subscription_id = self.get_subscription_id()
        artifact_source_name = resource_group + "ArtifactSource"
        updated_artifact_source_name = resource_group + "ArtifactSourceUpdated"
        location = resource_location

        self.assertFalse(location is None)

        self.set_managed_identity(subscription_id, resource_group)

        artifact_source_id = self.new_artifact_source(
            resource_group,
            storage_account,
            artifact_source_name,
            location,
            True)

        self.service_topology_validations(
            subscription_id,
            resource_group,
            location,
            artifact_source_id,
            updated_artifact_source_name,
            storage_account)

        self.list_artifact_sources(resource_group, artifact_source_name)

        self.delete_artifact_source(resource_group, artifact_source_name)
        self.delete_artifact_source(resource_group, updated_artifact_source_name)

    def service_topology_validations(
            self,
            subscription_id,
            resource_group_name,
            location,
            artifact_source_id,
            updated_artifact_source_name,
            storage_account_name):

        topology_name = resource_group_name + "ServiceTopology"
        topology2_name = resource_group_name + "ServiceTopology2"

        self.kwargs = {
            'rg': resource_group_name,
            'location': location,
            'st_name': topology_name,
            'as_id': artifact_source_id,
        }

        self.cmd('deploymentmanager service-topology create -g {rg} -n {st_name} -l \"{location}\" --artifact-source {as_id}', checks=[
            self.check('type', 'Microsoft.DeploymentManager/serviceTopologies'),
            self.check('name', topology_name),
            self.check('artifactSourceId', artifact_source_id)])

        service_topology_id = self.cmd('deploymentmanager service-topology show -g {rg} -n {st_name}').get_output_in_json()['id']
        self.services_validations(
            resource_group_name,
            location,
            artifact_source_id,
            service_topology_id,
            topology_name,
            subscription_id
        )

        updated_artifact_source_id = self.new_artifact_source(
            resource_group_name,
            storage_account_name,
            updated_artifact_source_name,
            location,
            False)

        self.kwargs = {
            'rg': resource_group_name,
            'location': location,
            'st_name': topology_name,
            'as_id': updated_artifact_source_id,
        }

        self.cmd('deploymentmanager service-topology update -g {rg} -n {st_name} --artifact-source {as_id}', checks=[
            self.check('type', 'Microsoft.DeploymentManager/serviceTopologies'),
            self.check('name', topology_name),
            self.check('artifactSourceId', updated_artifact_source_id)])

        self.kwargs = {
            'rg': resource_group_name,
            'location': location,
            'st_name': topology2_name,
            'as_id': artifact_source_id,
        }

        self.cmd('deploymentmanager service-topology create -g {rg} -n {st_name} -l \"{location}\" --artifact-source {as_id}', checks=[
            self.check('type', 'Microsoft.DeploymentManager/serviceTopologies'),
            self.check('name', topology2_name),
            self.check('artifactSourceId', artifact_source_id)])

        self.list_service_topologies(resource_group_name, topology_name)

        self.kwargs = {
            'rg': resource_group_name,
            'location': location,
            'st_name': topology_name,
            'as_id': artifact_source_id,
        }

        self.cmd('deploymentmanager service-topology delete -g {rg} -n {st_name}')
        with self.assertRaisesRegexp(SystemExit, '3'):
            self.cmd('deploymentmanager service-topology show -n {st_name} -g {rg}')

        self.kwargs = {
            'rg': resource_group_name,
            'location': location,
            'st_name': topology2_name,
            'as_id': artifact_source_id,
        }

        self.cmd('deploymentmanager service-topology delete -g {rg} -n {st_name}')
        with self.assertRaisesRegexp(SystemExit, '3'):
            self.cmd('deploymentmanager service-topology show -n {st_name} -g {rg}')

    def services_validations(
            self,
            resource_group_name,
            location,
            artifact_source_id,
            service_topology_id,
            service_topology_name,
            subscription_id):

        service_name = resource_group_name + "Service"
        service2_name = resource_group_name + "Service2"

        self.kwargs = {
            'rg': resource_group_name,
            'location': location,
            'st_name': service_topology_name,
            's_name': service_name,
            's2_name': service2_name,
            't_l': location,
            't_sub_id': subscription_id
        }

        self.cmd('deploymentmanager service create -g {rg} --service-topology-name {st_name} -n {s_name} -l \"{location}\" --target-location \"{t_l}\" --target-subscription-id {t_sub_id}', checks=[
            self.check('type', 'Microsoft.DeploymentManager/serviceTopologies/services'),
            self.check('name', service_name),
            self.check('targetLocation', location)])
        self.service_units_validations(
            resource_group_name,
            location,
            artifact_source_id,
            service_topology_id,
            service_topology_name,
            service_name
        )

        # dummy subscription id created for testing. No resources are created in this.
        updated_target_subscription_id = "29843263-a568-4db8-899f-10977b9d5c7b"
        self.kwargs = {
            'rg': resource_group_name,
            'location': location,
            'st_name': service_topology_name,
            's_name': service_name,
            's2_name': service2_name,
            't_l': location,
            't_sub_id': updated_target_subscription_id
        }

        self.cmd('deploymentmanager service update -g {rg} --service-topology-name {st_name} -n {s_name} --target-subscription-id {t_sub_id}', checks=[
            self.check('type', 'Microsoft.DeploymentManager/serviceTopologies/services'),
            self.check('name', service_name),
            self.check('targetSubscriptionId', updated_target_subscription_id)])

        self.cmd('deploymentmanager service create -g {rg} --service-topology-name {st_name} -n {s2_name} -l \"{location}\" --target-location \"{t_l}\" --target-subscription-id {t_sub_id}', checks=[
            self.check('type', 'Microsoft.DeploymentManager/serviceTopologies/services'),
            self.check('name', service2_name),
            self.check('targetLocation', location)])

        self.list_services(resource_group_name, service_topology_name, service_name)

        self.kwargs = {
            'rg': resource_group_name,
            'st_name': service_topology_name,
            's_name': service_name,
            's2_name': service2_name,
        }

        self.cmd('deploymentmanager service delete -g {rg} --service-topology-name {st_name} -n {s_name}')
        with self.assertRaisesRegexp(SystemExit, '3'):
            self.cmd('deploymentmanager service show -g {rg} --service-topology-name {st_name} -n {s_name}')

        self.cmd('deploymentmanager service delete -g {rg} --service-topology-name {st_name} -n {s2_name}')
        with self.assertRaisesRegexp(SystemExit, '3'):
            self.cmd('deploymentmanager service show -g {rg} --service-topology-name {st_name} -n {s2_name}')

    def service_units_validations(
            self,
            resource_group_name,
            location,
            artifact_source_id,
            service_topology_id,
            service_topology_name,
            service_name):

        service_unit_name = resource_group_name + "ServiceUnit"
        deployment_mode = "Incremental"
        self.kwargs = {
            'rg': resource_group_name,
            'location': location,
            'st_name': service_topology_name,
            's_name': service_name,
            'su_name': service_unit_name,
            'd_mode': deployment_mode,
            'p_path': parameters_file_name,
            't_path': template_file_name
        }

        self.cmd('deploymentmanager service-unit create -g {rg} --service-topology-name {st_name} --service-name {s_name} -n {su_name} -l \"{location}\" --target-resource-group {rg} --deployment-mode {d_mode} --parameters-path {p_path} --template-path {t_path}', checks=[
            self.check('type', 'Microsoft.DeploymentManager/serviceTopologies/services/serviceUnits'),
            self.check('name', service_unit_name),
            self.check('artifacts.parametersArtifactSourceRelativePath', parameters_file_name),
            self.check('artifacts.templateArtifactSourceRelativePath', template_file_name),
            self.check('deploymentMode', deployment_mode),
            self.check('targetResourceGroup', resource_group_name)])

        service_unit_id = self.cmd('deploymentmanager service-unit show -g {rg} --service-topology-name {st_name} --service-name {s_name} -n {su_name}').get_output_in_json()['id']

        # Create a service unit that is invalid to test rollout failure operations
        invalid_service_unit_name = resource_group_name + "InvalidServiceUnit"
        self.kwargs = {
            'rg': resource_group_name,
            'location': location,
            'st_name': service_topology_name,
            's_name': service_name,
            'su_name': invalid_service_unit_name,
            'd_mode': deployment_mode,
            'p_path': invalid_parameters_file_name,
            't_path': template_file_name
        }

        self.cmd('deploymentmanager service-unit create -g {rg} --service-topology-name {st_name} --service-name {s_name} -n {su_name} -l \"{location}\" --target-resource-group {rg} --deployment-mode {d_mode} --parameters-path {p_path} --template-path {t_path}', checks=[
            self.check('type', 'Microsoft.DeploymentManager/serviceTopologies/services/serviceUnits'),
            self.check('name', invalid_service_unit_name),
            self.check('artifacts.parametersArtifactSourceRelativePath', invalid_parameters_file_name),
            self.check('artifacts.templateArtifactSourceRelativePath', template_file_name),
            self.check('deploymentMode', deployment_mode),
            self.check('targetResourceGroup', resource_group_name)])

        invalid_service_unit_id = self.cmd('deploymentmanager service-unit show -g {rg} --service-topology-name {st_name} --service-name {s_name} -n {su_name}').get_output_in_json()['id']
        self.steps_validations(
            resource_group_name,
            location,
            artifact_source_id,
            service_topology_id,
            service_topology_name,
            service_unit_id,
            invalid_service_unit_id)

        deployment_mode = "Complete"
        self.kwargs = {
            'rg': resource_group_name,
            'location': location,
            'st_name': service_topology_name,
            's_name': service_name,
            'su_name': service_unit_name,
            'd_mode': deployment_mode,
            'p_path': parameters_file_name,
            't_path': template_file_name
        }

        self.cmd('deploymentmanager service-unit update -g {rg} --service-topology-name {st_name} --service-name {s_name} -n {su_name} --deployment-mode {d_mode}', checks=[
            self.check('type', 'Microsoft.DeploymentManager/serviceTopologies/services/serviceUnits'),
            self.check('name', service_unit_name),
            self.check('deploymentMode', deployment_mode)])

        self.list_service_units(resource_group_name, service_topology_name, service_name, service_unit_name)

        self.kwargs = {
            'rg': resource_group_name,
            'st_name': service_topology_name,
            's_name': service_name,
            'su_name': service_unit_name,
        }

        self.cmd('deploymentmanager service-unit delete -g {rg} --service-topology-name {st_name} --service-name {s_name} -n {su_name}')
        with self.assertRaisesRegexp(SystemExit, '3'):
            self.cmd('deploymentmanager service-unit show -g {rg} --service-topology-name {st_name} --service-name {s_name} -n {su_name}')

        self.kwargs = {
            'rg': resource_group_name,
            'st_name': service_topology_name,
            's_name': service_name,
            'su_name': invalid_service_unit_name,
        }

        self.cmd('deploymentmanager service-unit delete -g {rg} --service-topology-name {st_name} --service-name {s_name} -n {su_name}')
        with self.assertRaisesRegexp(SystemExit, '3'):
            self.cmd('deploymentmanager service-unit show -g {rg} --service-topology-name {st_name} --service-name {s_name} -n {su_name}')

    def steps_validations(
            self,
            resource_group_name,
            location,
            artifact_source_id,
            service_topology_id,
            service_topology_name,
            service_unit_id,
            invalid_service_unit_id):

        step_name = resource_group_name + "WaitStep"
        duration = "PT5M"
        updated_duration = "PT10M"

        self.kwargs = {
            'rg': resource_group_name,
            'location': location,
            'step_name': step_name,
            'duration': duration
        }

        self.cmd('deploymentmanager step create -g {rg} -l \"{location}\" -n {step_name} --duration {duration}', checks=[
            self.check('type', 'Microsoft.DeploymentManager/steps'),
            self.check('name', step_name),
            self.check('properties.stepType', 'Wait'),
            self.check('properties.attributes.duration', duration)])

        step_id = self.cmd('deploymentmanager step show -g {rg} -n {step_name}').get_output_in_json()['id']

        self.rollouts_validations(
            resource_group_name,
            location,
            artifact_source_id,
            service_topology_id,
            service_unit_id,
            invalid_service_unit_id,
            step_id
        )

        self.kwargs = {
            'rg': resource_group_name,
            'location': location,
            'step_name': step_name,
            'duration': updated_duration
        }

        self.cmd('deploymentmanager step update -g {rg} -n {step_name} --duration {duration}', checks=[
            self.check('type', 'Microsoft.DeploymentManager/steps'),
            self.check('name', step_name),
            self.check('properties.attributes.duration', updated_duration)])

        self.healthcheck_step_validations(resource_group_name, location)

        self.kwargs = {
            'rg': resource_group_name,
            'step_name': step_name
        }

        self.cmd('deploymentmanager step delete -g {rg} -n {step_name}')
        with self.assertRaisesRegexp(SystemExit, '3'):
            self.cmd('deploymentmanager step show -g {rg} -n {step_name}')

    def healthcheck_step_validations(
            self,
            resource_group_name,
            location):

        step_name = resource_group_name + "RestHealthCheckStep"
        updated_healthy_state_duration = "PT30M"

        self.replace_string("__HEALTH_CHECK_STEP_NAME__", step_name, healthcheck_file_path, True)

        self.kwargs = {
            'rg': resource_group_name,
            'step_name': step_name,
            'rest_health_check_file': healthcheck_file_path,
            'healthy_state_duration': updated_healthy_state_duration
        }

        json_obj = None
        with open(healthcheck_file_path) as f:
            json_obj = json.load(f)

        self.cmd('deploymentmanager step create -g {rg} --step "{rest_health_check_file}"', checks=[
            self.check('type', 'Microsoft.DeploymentManager/steps'),
            self.check('name', step_name),
            self.check('properties.stepType', 'HealthCheck'),
            self.check('properties.attributes.waitDuration', json_obj['properties']['attributes']['waitDuration']),
            self.check('properties.attributes.maxElasticDuration', json_obj['properties']['attributes']['maxElasticDuration']),
            self.check('properties.attributes.healthyStateDuration', json_obj['properties']['attributes']['healthyStateDuration'])])

        step_id = self.cmd('deploymentmanager step show -g {rg} -n {step_name}').get_output_in_json()['id']
        self.assertFalse(step_id is None)

        # Revert health check file change for playback mode.
        self.replace_string(step_name, "__HEALTH_CHECK_STEP_NAME__", healthcheck_file_path, True)

        json_obj['properties']['attributes']['healthyStateDuration'] = updated_healthy_state_duration

        serialized_json = json.dumps(json_obj)
        self.assertFalse(serialized_json is None)

        self.kwargs = {
            'rg': resource_group_name,
            'step_name': step_name,
            'step': serialized_json
        }

        self.cmd('deploymentmanager step update -g {rg} -n {step_name} --step \'{step}\'', checks=[
            self.check('type', 'Microsoft.DeploymentManager/steps'),
            self.check('name', step_name),
            self.check('properties.attributes.healthyStateDuration', updated_healthy_state_duration)])

        step2_name = resource_group_name + "RestHealthCheckStep2"
        json_obj['name'] = step2_name

        serialized_json = json.dumps(json_obj)
        self.assertFalse(serialized_json is None)

        self.kwargs = {
            'rg': resource_group_name,
            'step': serialized_json
        }

        self.cmd('deploymentmanager step create -g {rg} --step \'{step}\'', checks=[
            self.check('type', 'Microsoft.DeploymentManager/steps'),
            self.check('name', step2_name),
            self.check('properties.stepType', 'HealthCheck'),
            self.check('properties.attributes.healthyStateDuration', updated_healthy_state_duration)])

        self.list_steps(resource_group_name, step_name)

        self.kwargs = {
            'rg': resource_group_name,
            'step_name': step_name
        }

        self.cmd('deploymentmanager step delete -g {rg} -n {step_name}')
        with self.assertRaisesRegexp(SystemExit, '3'):
            self.cmd('deploymentmanager step show -g {rg} -n {step_name}')

        self.kwargs = {
            'rg': resource_group_name,
            'step2_name': step2_name
        }

        self.cmd('deploymentmanager step delete -g {rg} -n {step2_name}')
        with self.assertRaisesRegexp(SystemExit, '3'):
            self.cmd('deploymentmanager step show -g {rg} -n {step2_name}')

    def rollouts_validations(
            self,
            resource_group_name,
            location,
            artifact_source_id,
            service_topology_id,
            service_unit_id,
            invalid_service_unit_id,
            step_id):

        rollout_name = resource_group_name + "Rollout"
        failed_rollout_name = resource_group_name + "InvalidRollout"

        self.replace_rollout_placeholders(
            rollout_name,
            service_topology_id,
            artifact_source_id,
            step_id,
            service_unit_id,
            create_rollout_template
        )

        self.replace_rollout_placeholders(
            failed_rollout_name,
            service_topology_id,
            artifact_source_id,
            step_id,
            invalid_service_unit_id,
            failure_create_rollout_template
        )

        self.kwargs = {
            'rg': resource_group_name,
            'location': location,
            't_path': create_rollout_template,
            'invalid_t_path': failure_create_rollout_template,
            'rollout_name': rollout_name,
            'failed_rollout_name': failed_rollout_name
        }

        self.cmd('group deployment create -g {rg} --template-file "{t_path}"', checks=[
            self.check('properties.provisioningState', 'Succeeded')])

        self.cmd('deploymentmanager rollout show -g {rg} -n {rollout_name}', checks=[
            self.check('type', 'Microsoft.DeploymentManager/rollouts'),
            self.check('status', 'Running'),
            self.check('totalRetryAttempts', '0'),
            self.check('operationInfo.retryAttempt', '0'),
            self.check('artifactSourceId', artifact_source_id),
            self.check('targetServiceTopologyId', service_topology_id)])

        self.cmd('deploymentmanager rollout stop -g {rg} -n {rollout_name} --yes', checks=[
            self.check('type', 'Microsoft.DeploymentManager/rollouts'),
            self.check('status', 'Canceling')])

        while True:
            self.sleep(120)
            rollout = self.cmd('deploymentmanager rollout show -g {rg} -n {rollout_name}').get_output_in_json()
            if (rollout['status'] != 'Canceling'):
                break

        self.assertEqual('Canceled', rollout['status'])

        self.cmd('group deployment create -g {rg} --template-file "{invalid_t_path}"', checks=[
            self.check('properties.provisioningState', 'Succeeded')])

        self.cmd('deploymentmanager rollout show -g {rg} -n {failed_rollout_name}', checks=[
            self.check('type', 'Microsoft.DeploymentManager/rollouts'),
            self.check('totalRetryAttempts', '0'),
            self.check('operationInfo.retryAttempt', '0'),
            self.check('artifactSourceId', artifact_source_id),
            self.check('targetServiceTopologyId', service_topology_id)])

        # Now validate that the rollout expected to fail has failed.
        while True:
            self.sleep(120)
            failed_rollout = self.cmd('deploymentmanager rollout show -g {rg} -n {failed_rollout_name}').get_output_in_json()
            if (failed_rollout['status'] != 'Running'):
                break

        self.assertEqual('Failed', failed_rollout['status'])

        self.cmd('deploymentmanager rollout restart -g {rg} -n {failed_rollout_name} --skip-succeeded --yes', checks=[
            self.check('type', 'Microsoft.DeploymentManager/rollouts'),
            self.check('operationInfo.retryAttempt', '1'),
            self.check('operationInfo.skipSucceededOnRetry', True),
            self.check('status', 'Running')])

        self.cmd('deploymentmanager rollout stop -g {rg} -n {failed_rollout_name} --yes', checks=[
            self.check('type', 'Microsoft.DeploymentManager/rollouts'),
            self.check('operationInfo.retryAttempt', '1'),
            self.check('operationInfo.skipSucceededOnRetry', True),
            self.check('status', 'Canceling')])

        while True:
            self.sleep(120)
            failed_rollout = self.cmd('deploymentmanager rollout show -g {rg} -n {failed_rollout_name}').get_output_in_json()
            if (failed_rollout['status'] != 'Canceling'):
                break

        self.cmd('deploymentmanager rollout delete -g {rg} -n {failed_rollout_name}')
        self.cmd('deploymentmanager rollout delete -g {rg} -n {rollout_name}')

        with self.assertRaisesRegexp(SystemExit, '3'):
            self.cmd('deploymentmanager rollout show -g {rg} -n {failed_rollout_name}')
        with self.assertRaisesRegexp(SystemExit, '3'):
            self.cmd('deploymentmanager rollout show -g {rg} -n {rollout_name}')

    def set_managed_identity(self, subscription_id, resource_group_name):

        is_playback = os.path.exists(self.recording_file)
        if not is_playback:
            identity_name = resource_group_name + "Identity"
            self.kwargs = {
                'rg': resource_group_name,
                'name': identity_name,
            }
            identity = self.cmd('identity create -n {name} -g {rg}').get_output_in_json()
            identityId = identity['id']

            self.sleep(120)

            self.kwargs = {
                'rg': resource_group_name,
                'principalId': identity['principalId'],
                'role': "Contributor",
                'scope': "/subscriptions/{0}".format(subscription_id),
            }

            self.cmd('role assignment create --assignee {principalId} --role {role} --scope {scope}')

            self.sleep(30)

            self.replace_string("__USER_ASSIGNED_IDENTITY__", identityId, create_rollout_template)
            self.replace_string("__USER_ASSIGNED_IDENTITY__", identityId, failure_create_rollout_template)

            return identityId

        return "dummyid"

    def new_artifact_source(
            self,
            resource_group_name,
            storage_account_name,
            artifact_source_name,
            location,
            setup_container):

        artifact_root = "artifactroot"
        container_name = "artifacts"

        sas_uri = self.get_sas_for_artifacts_container(
            resource_group_name,
            storage_account_name,
            container_name,
            artifact_root,
            setup_container)

        self.kwargs = {
            'rg': resource_group_name,
            'location': location,
            'as_name': artifact_source_name,
            'sas': sas_uri,
            'artifactroot': artifact_root
        }

        self.cmd('deploymentmanager artifact-source create -g {rg} -n {as_name} -l \"{location}\" --sas-uri \"{sas}\" --artifact-root {artifactroot}', checks=[
            self.check('type', 'Microsoft.DeploymentManager/artifactSources'),
            self.check('artifactRoot', artifact_root),
            self.check('name', artifact_source_name)
        ])

        return self.cmd('deploymentmanager artifact-source show -g {rg} -n {as_name}').get_output_in_json()['id']

    def get_sas_for_artifacts_container(
            self,
            resource_group_name,
            storage_account_name,
            storage_container_name,
            artifact_root,
            setup_container):

        is_playback = os.path.exists(self.recording_file)
        if not is_playback:
            if setup_container:
                self.setup_artifacts_container(resource_group_name, storage_account_name, storage_container_name)

            return self.create_sas_key_for_container(storage_account_name, storage_container_name)

        return "dummysasforcontainer"

    def create_sas_key_for_container(self, storage_account_name, container_name):
        from datetime import datetime, timedelta
        start = (datetime.utcnow() + timedelta(hours=-1)).strftime('%Y-%m-%dT%H:%MZ')
        expiry = (datetime.utcnow() + timedelta(hours=10)).strftime('%Y-%m-%dT%H:%MZ')
        self.kwargs = {
            'account': storage_account_name,
            'container': container_name,
            'expiry': expiry,
            'start': start
        }

        sas = self.cmd('storage container generate-sas -n {container} --account-name {account} '
                       '--permissions rl --start {start} --expiry {expiry} -otsv').output.strip()

        full_uri = "https://" + storage_account_name + ".blob.core.windows.net/" + container_name + "?" + sas
        return full_uri

    def setup_artifacts_container(self, resource_group_name, storage_account_name, container_name):
        stgAcctForTemplate = resource_group_name + "stgtemplate"
        storageAcountReplacementSymbol = "__STORAGEACCOUNTNAME__"

        self.replace_string(storageAcountReplacementSymbol, stgAcctForTemplate, parametersArtifactSourceRelativePath)
        self.replace_string(storageAcountReplacementSymbol, stgAcctForTemplate, parametersCopyArtifactSourceRelativePath)

        storage_account_info = self.get_stg_account_info(resource_group_name, storage_account_name)
        self.create_container(storage_account_info, container_name)

        self.upload_blob(storage_account_info, container_name, parametersArtifactSourceRelativePath, parameters_file_name)
        self.upload_blob(storage_account_info, container_name, parametersCopyArtifactSourceRelativePath, params_copy_file_name)
        self.upload_blob(storage_account_info, container_name, templateArtifactSourceRelativePath, template_file_name)
        self.upload_blob(storage_account_info, container_name, templateCopyArtifactSourceRelativePath, template_copy_file_name)
        self.upload_blob(storage_account_info, container_name, invalidParametersArtifactSourceRelativePath, invalid_parameters_file_name)

    def list_steps(self, resource_group_name, step_name):
        self.kwargs = {
            'rg': resource_group_name,
            'step_name': step_name
        }

        steps = self.cmd('deploymentmanager step list -g {rg}', checks=[
            self.check("length(@)", 3)
        ]).get_output_in_json()

        selected_step = [x for x in steps if(x['name'].lower() == step_name.lower())]
        self.assertIsNotNone(selected_step, "list steps did not return {step_name}.")

    def list_service_units(self, resource_group_name, service_topology_name, service_name, service_unit_name):
        self.kwargs = {
            'rg': resource_group_name,
            'st_name': service_topology_name,
            's_name': service_name,
            'su_name': service_unit_name
        }

        service_units = self.cmd('deploymentmanager service-unit list -g {rg} --service-topology-name {st_name} --service-name {s_name}', checks=[
            self.check("length(@)", 2)
        ]).get_output_in_json()

        selected_service_unit = [x for x in service_units if(x['name'].lower() == service_unit_name.lower())]
        self.assertIsNotNone(selected_service_unit, "list service units did not return {su_name}.")

    def list_services(self, resource_group_name, service_topology_name, service_name):
        self.kwargs = {
            'rg': resource_group_name,
            'st_name': service_topology_name
        }

        services = self.cmd('deploymentmanager service list -g {rg} --service-topology-name {st_name}', checks=[
            self.check("length(@)", 2)
        ]).get_output_in_json()

        self.kwargs = {
            's_name': service_name
        }

        selected_service = [x for x in services if(x['name'].lower() == service_name.lower())]
        self.assertIsNotNone(selected_service, "list services did not return {s_name}.")

    def list_service_topologies(self, resource_group_name, service_topology_name):
        self.kwargs = {
            'rg': resource_group_name
        }

        service_topologies = self.cmd('deploymentmanager service-topology list -g {rg}', checks=[
            self.check("length(@)", 2)
        ]).get_output_in_json()

        self.kwargs = {
            'st_name': service_topology_name
        }

        selected_service_topology = [x for x in service_topologies if(x['name'].lower() == service_topology_name.lower())]
        self.assertIsNotNone(selected_service_topology, "list service topologies did not return {st_name}.")

    def list_artifact_sources(self, resource_group_name, artifact_source_name):
        self.kwargs = {
            'rg': resource_group_name
        }

        artifact_sources = self.cmd('deploymentmanager artifact-source list -g {rg}', checks=[
            self.check("length(@)", 2)
        ]).get_output_in_json()

        self.kwargs = {
            'as_name': artifact_source_name
        }

        selected_artifact_source = [x for x in artifact_sources if(x['name'].lower() == artifact_source_name.lower())]
        self.assertIsNotNone(selected_artifact_source, "list artifact sources did not return {as_name}.")

    def delete_artifact_source(self, resource_group_name, artifact_source_name):
        self.kwargs = {
            'rg': resource_group_name,
            'name': artifact_source_name,
        }
        self.cmd('deploymentmanager artifact-source delete -n {name} -g {rg} --yes')

        with self.assertRaisesRegexp(SystemExit, '3'):
            self.cmd('deploymentmanager artifact-source show -n {name} -g {rg}')

    def upload_blob(self, storage_account_info, storage_container_name, file_path, file_name):
        abs_file_path = os.path.join(TEST_DIR, file_path).replace('\\', '\\\\')
        blob_path = artifact_root + "/" + file_name
        self.storage_cmd('storage blob upload -c {} -n {} -f "{}" -t block', storage_account_info,
                         storage_container_name, blob_path, abs_file_path)

    def get_stg_account_key(self, group, name):

        template = 'storage account keys list -n {} -g {} --query "[0].value" -otsv'
        return self.cmd(template.format(name, group)).output

    def get_stg_account_info(self, group, name):
        """Returns the storage account name and key in a tuple"""
        return name, self.get_stg_account_key(group, name)

    def create_container(self, account_info, container_name):
        self.storage_cmd('storage container create -n {}', account_info, container_name)
        return container_name

    def storage_cmd(self, cmd, account_info, *args):
        cmd = cmd.format(*args)
        cmd = '{} --account-name {} --account-key {}'.format(cmd, *account_info)
        return self.cmd(cmd)

    def replace_string(self, replacement_symbol, replacement_value, filePath, overridePlayback=False):
        is_playback = os.path.exists(self.recording_file)
        if (not is_playback or overridePlayback):
            file = fileinput.FileInput(filePath, inplace=True)
            for line in file:
                print(line.replace(replacement_symbol, replacement_value), end='')
            file.close()

    def replace_rollout_placeholders(
        self,
        rollout_name,
        target_service_topology_id,
        artifact_source_id,
        step_id,
        service_unit_id,
        file_path
    ):
        is_playback = os.path.exists(self.recording_file)
        if not is_playback:
            self.replace_string("__ROLLOUT_NAME__", rollout_name, file_path)
            self.replace_string("__TARGET_SERVICE_TOPOLOGY__", target_service_topology_id, file_path)
            self.replace_string("__ARTIFACT_SOURCE_ID__", artifact_source_id, file_path)
            self.replace_string("__STEP_ID__", step_id, file_path)
            self.replace_string("__SERVICE_UNIT_ID__", service_unit_id, file_path)

    def sleep(self, duration):
        is_playback = os.path.exists(self.recording_file)
        if not is_playback:
            time.sleep(duration)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from importlib import import_module

DUMMY_CLI_TYPE = 'DummyCli'
DUMMY_CLI_MODULE = 'azure.cli.core.mock'


def get_dummy_cli(*args, **kwargs):
    mod = import_module('azure.cli.core.mock')
    return getattr(mod, 'DummyCli')(*args, **kwargs)


def get_support_api_version_func():
    mod = import_module('azure.cli.core.profiles')
    return getattr(mod, 'supported_api_version')


def get_commands_loggers():
    mod = import_module('azure.cli.core.commands')
    return getattr(mod, 'logger')
